package com.arms.api.analysis.cost.service;

import com.arms.api.analysis.cost.dto.CostDTO;
import com.arms.api.analysis.cost.vo.CostVO;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.util.model.dto.PdServiceAndIsReqDTO;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.ExistsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.model.dto.집계_하위_필드;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentBucket;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class AnalysisCostImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements AnalysisCost {

    @Override
    public List<CostVO> getAssigneeListByProductVersionAndRequirement(CostDTO dto) {

        PdServiceAndIsReqDTO pdServiceAndIsReq = dto.getPdServiceAndIsReq();

        boolean isReq = pdServiceAndIsReq.getIsReq();

        dto.set크기(10000);
        dto.set하위_크기(10000);
        dto.set메인_그룹_필드("pdServiceVersions");
        dto.set메인_그룹_필드명_별칭("pdServiceVersions");
        dto.set집계_하위_필드들(List.of(
                        집계_하위_필드.builder()
                                .하위_필드명_별칭(isReq ? "requirement" : "parentRequirement")
                                .하위_필드명(isReq ? "key" : "parentReqKey")
                                .크기(dto.get크기())
                                .build(),
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assignees")
                                .하위_필드명("assignee.assignee_accountId.keyword")
                                .크기(dto.get크기())
                                .결과_갯수_기준_오름차순(false)
                                .build(),
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("displayNames")
                                .하위_필드명("assignee.assignee_displayName.keyword")
                                .크기(dto.get크기())
                                .build()
                )
        );


        EsQuery esQuery = new EsQueryBuilder()
                .bool(new TermQueryMust("pdServiceId", dto.getPdServiceLink()),
                        new TermQueryMust("isReq", isReq),
                        new TermsQueryFilter("pdServiceVersions", dto.getPdServiceVersionLinks()),
                        new ExistsQueryFilter("assignee")
                );

        dto.setEsQuery(esQuery);

        DocumentAggregations aggregations = this.aggregateRecentDocs(dto);
        List<DocumentBucket> subList = aggregations.deepestSubList();

        List<CostVO> costVOList = new ArrayList<>();
        for (DocumentBucket docBucket : subList) {
            String versionKey = docBucket.getDocFieldName("pdServiceVersions");
            Long versionCount = docBucket.getDocFieldNameCount("pdServiceVersions");
            String reqKey = docBucket.getDocFieldName("requirement");
            Long reqCount = docBucket.getDocFieldNameCount("requirement");
            String subReqKey = docBucket.getDocFieldName("parentRequirement");
            Long subReqCount = docBucket.getDocFieldNameCount("parentRequirement");
            String assigneeKey = docBucket.getDocFieldName("assignees");
            Long assigneeCount = docBucket.getDocFieldNameCount("assignees");
            String displayNameKey = docBucket.getDocFieldName("displayNames");
            Long displayNameCount = docBucket.getDocFieldNameCount("displayNames");
            CostVO costVO = CostVO.builder()
                    .versionKey(versionKey)
                    .versionCount(versionCount)
                    .reqKey(reqKey)
                    .reqCount(reqCount)
                    .subReqKey(subReqKey)
                    .subReqCount(subReqCount)
                    .assigneeKey(assigneeKey)
                    .assigneeCount(assigneeCount)
                    .displayNameKey(displayNameKey)
                    .displayNameCount(displayNameCount)
                    .build();
            costVOList.add(costVO);
        }


        return costVOList;
    }


    @Override
    public List<CostVO> aggregationByAssigneeAccountId(CostDTO dto) {

        EsQuery esQuery = new EsQueryBuilder()
                .bool(new TermQueryMust("pdServiceId", dto.getPdServiceLink()),
                        new TermsQueryFilter("pdServiceVersions", dto.getPdServiceVersionLinks()),
                        new ExistsQueryFilter("assignee")
                );

        dto.set크기(10000);
        dto.set하위_크기(10000);
        dto.set메인_그룹_필드("assignee.assignee_accountId.keyword");
        dto.set메인_그룹_필드명_별칭("accountId");
        dto.setEsQuery(esQuery);

        DocumentAggregations aggregations = this.aggregateRecentDocs(dto);

        List<DocumentBucket> documentBuckets = aggregations.docBuckets();

        List<CostVO> costVOList = new ArrayList<>();

        for (DocumentBucket documentBucket : documentBuckets) {
            costVOList.add(CostVO.builder()
                    .assigneeKey(documentBucket.getDocFieldName("accountId"))
                    .assigneeCount(documentBucket.getDocFieldNameCount("accountId"))
                    .build());
        }

        return costVOList;
    }

    @Override
    public List<CostVO> aggregationByReqLinkAndAssigneeAccountId(CostDTO dto) {
        EsQuery esQuery = new EsQueryBuilder()
                .bool(new TermQueryMust("pdServiceId", dto.getPdServiceLink()),
                        new TermsQueryFilter("pdServiceVersions", dto.getPdServiceVersionLinks()),
                        new TermQueryMust("isReq", dto.getIsReq())
                );

        dto.set크기(10000);
        dto.set하위_크기(10000);
        dto.set메인_그룹_필드("cReqLink");
        dto.set메인_그룹_필드명_별칭("cReqLink");
        dto.setEsQuery(esQuery);

        dto.set집계_하위_필드들(List.of(
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assignees")
                                .하위_필드명("assignee.assignee_accountId.keyword")
                                .크기(dto.get크기())
                                .결과_갯수_기준_오름차순(false)
                                .build()
                )
        );

        DocumentAggregations aggregations = this.aggregateRecentDocs(dto);

        List<DocumentBucket> documentBuckets = aggregations.deepestSubList();

        List<CostVO> costVOList = new ArrayList<>();

        for (DocumentBucket documentBucket : documentBuckets) {
            String cReqLinkKey = documentBucket.getDocFieldName("cReqLink");
            Long cReqLinkCount = documentBucket.getDocFieldNameCount("cReqLink");
            String assigneeKey = documentBucket.getDocFieldName("assignees");
            Long assigneeCount = documentBucket.getDocFieldNameCount("assignees");

            CostVO costVO = CostVO.builder()
                    .reqLinkKey(cReqLinkKey)
                    .reqLinkCount(cReqLinkCount)
                    .assigneeKey(assigneeKey)
                    .assigneeCount(assigneeCount)
                    .build();
            costVOList.add(costVO);
        }

        return costVOList;
    }

}
