package com.arms.api.analysis.requirement.service;

import com.arms.api.analysis.requirement.dto.ReqAnalysisAggrDTO;
import com.arms.api.analysis.requirement.dto.ReqAnalysisDTO;
import com.arms.api.analysis.requirement.vo.ReqAndIssueTotalCountVO;
import com.arms.api.analysis.requirement.vo.ReqProgressVO;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.issue.almapi.repository.지라이슈_저장소;
import com.arms.api.util.model.dto.PdServiceAndIsReqDTO;
import com.arms.api.util.model.dto.response.*;
import com.arms.api.util.model.enums.IsReqType;
import com.arms.api.util.model.dto.request.ALM_이슈_제품_및_버전_집계_요청;
import com.arms.api.util.model.dto.request.ALM_이슈_일별_제품_및_버전_집계_요청;
import com.arms.egovframework.javaservice.esframework.model.dto.*;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsBoolQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.EsSortQuery;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.factory.creator.기본_쿼리_생성기;
import com.arms.egovframework.javaservice.esframework.factory.creator.중첩_집계_쿼리_생성기;
import com.arms.egovframework.javaservice.esframework.esquery.filter.ExistsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.RangeQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentBucket;
import com.arms.egovframework.javaservice.esframework.model.vo.버킷_집계_결과;
import com.arms.egovframework.javaservice.esframework.model.vo.버킷_집계_결과_목록_합계;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static com.arms.api.util.model.enums.IsReqType.*;
import static java.util.stream.Collectors.toList;

@Slf4j
@Service("요구사항_분석_서비스")
@AllArgsConstructor
public class ReqAnalysisServiceImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements ReqAnalysisService {

    private final 지라이슈_저장소 지라이슈_저장소;
    @Override
    public ReqAndIssueTotalCountVO topMenuTotalCountWithReqAndSubtask(ReqAnalysisDTO reqAnalysisDTO) {

        ReqAnalysisAggrDTO reqAnalysisAggrDTO = new ReqAnalysisAggrDTO();
        reqAnalysisAggrDTO.setPdServiceAndIsReq(reqAnalysisDTO.getPdServiceAndIsReq());
        PdServiceAndIsReqDTO pdServiceAndIsReq = reqAnalysisAggrDTO.getPdServiceAndIsReq();

        reqAnalysisAggrDTO.set메인_그룹_필드("isReq");
        reqAnalysisAggrDTO.set메인_그룹_필드명_별칭("isReq");
        reqAnalysisAggrDTO.set크기(10000);

        EsQuery esQuery
            = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("pdServiceId",pdServiceAndIsReq.getPdServiceId()),
                        new TermsQueryFilter("pdServiceVersions",pdServiceAndIsReq.getPdServiceVersions())
                );

        reqAnalysisAggrDTO.setEsQuery(esQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(reqAnalysisAggrDTO);

        List<DocumentBucket> docBuckets = documentAggregations.docBuckets();

        return ReqAndIssueTotalCountVO.builder()
            .total(documentAggregations.getTotalHits())
            .req(
                docBuckets.stream().filter(a -> "true".equals(a.getDocFieldName("isReq")))
                    .map(a -> a.getDocFieldNameCount("isReq"))
                    .findFirst().orElse(0L)
            )
            .subtask(
                docBuckets.stream().filter(a -> "false".equals(a.getDocFieldName("isReq")))
                    .map(a -> a.getDocFieldNameCount("isReq"))
                    .findFirst().orElse(0L))
            .build();
    }

    @Override
    public List<ReqProgressVO> 일반_버전필터_해결책유무_검색(
            ReqAnalysisDTO reqAnalysisDTO, String resolution) {

        ReqAnalysisAggrDTO reqAnalysisAggrDTO = new ReqAnalysisAggrDTO();
        reqAnalysisAggrDTO.setPdServiceAndIsReq(reqAnalysisDTO.getPdServiceAndIsReq());
        PdServiceAndIsReqDTO pdServiceAndIsReq = reqAnalysisAggrDTO.getPdServiceAndIsReq();

        reqAnalysisAggrDTO.set메인_그룹_필드("cReqLink");
        reqAnalysisAggrDTO.set메인_그룹_필드명_별칭("cReqLink");

        EsQuery esQuery
                = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", pdServiceAndIsReq.getPdServiceId())
                        , new TermQueryMust("isReq", pdServiceAndIsReq.getIsReq())
                        , new TermsQueryFilter("pdServiceVersions", pdServiceAndIsReq.getPdServiceVersions())
                        , new ExistsQueryFilter(resolution) // resolution 이 null 일때는 Exists 작동 안해야함.
                );

        reqAnalysisAggrDTO.setEsQuery(esQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(reqAnalysisAggrDTO);

        List<DocumentBucket> documentBuckets = documentAggregations.deepestSubList();

        return buildReqProgressList(documentBuckets);
    }

    private List<ReqProgressVO> buildReqProgressList(List<DocumentBucket> documentBuckets) {
        List<ReqProgressVO> reqProgressList = new ArrayList<>();

        if (documentBuckets.isEmpty()) {
            return Collections.emptyList();
        }

        for (DocumentBucket documentBucket : documentBuckets) {
            reqProgressList.add(ReqProgressVO.builder()
                    .cReqLink(Long.valueOf(documentBucket.getDocFieldName("cReqLink")))
                    .docCount(documentBucket.getDocCount())
                    .build());
        }

        return  reqProgressList;
    }


    @Override
    public Map<String, 요구사항_지라이슈상태_주별_집계> 요구사항_지라이슈상태_주별_집계(ALM_이슈_제품_및_버전_집계_요청 지라이슈_제품_및_제품버전_집계_요청) {
        LocalDate now = LocalDate.now(ZoneId.of("UTC"));
        LocalDate monthAgo = now.minusWeeks(4);

        // 1. 검색 범위에 포함되지 않은 누적 데이터를 가져온다. 기간이 길어지면 길어질수록 무의미한, 반복적인 연산이기 때문에 캐싱 고려일별_생성개수_및_상태_데이터생성
        요구사항_지라이슈상태_주별_집계 HistoricalData = 누적데이터조회(
                지라이슈_제품_및_제품버전_집계_요청.getPdServiceLink(),
                지라이슈_제품_및_제품버전_집계_요청.getPdServiceVersionLinks(),
                monthAgo
        );
        Map<String, Long> totalStatuses = new HashMap<>(HistoricalData.getStatuses());
        long totalIssues = HistoricalData.getTotalIssues();
        long totalRequirements = HistoricalData.getTotalRequirements();

        // 2. 검색 범위 내의 데이터를 가져온다. 현재 검색 범위는 차트 UI를 고려하여, 4~5주 정도로 적용
        EsQuery esQuery = new EsQueryBuilder()
                .bool(new TermQueryMust("pdServiceId", 지라이슈_제품_및_제품버전_집계_요청.getPdServiceLink()),
                        new TermsQueryFilter("pdServiceVersions", 지라이슈_제품_및_제품버전_집계_요청.getPdServiceVersionLinks()),
                        new TermQueryMust("isReq", true),
                        new RangeQueryFilter("created", monthAgo, now, "fromto")
                );
        기본_검색_집계_하위_요청 하위_집계_요청 = new 기본_검색_집계_하위_요청(){};
        하위_집계_요청.set메인_그룹_필드("created");
        하위_집계_요청.set집계_하위_필드들(
            List.of(
                집계_하위_필드.builder()
                    .하위_필드명("status.status_name.keyword")
                    .build())
        );

        버킷_집계_결과_목록_합계 버킷_집계_결과_목록_합계 = 지라이슈_저장소.aggregateRecentDocs(중첩_집계_쿼리_생성기.포괄_주별(하위_집계_요청,esQuery).생성());

        List<버킷_집계_결과> aggregationByWeek = 버킷_집계_결과_목록_합계.get검색결과().get("date_group_by_"+하위_집계_요청.get메인_그룹_필드());


        Map<String, 요구사항_지라이슈상태_주별_집계> 검색결과 = aggregationByWeek.stream()
                .sorted(Comparator.comparing(bucket -> OffsetDateTime.parse(bucket.get필드명()).toLocalDate()))
                .collect(Collectors.toMap(
                        entry -> transformDate(entry.get필드명()),
                        this::주별데이터생성,
                        (existingValue, newValue) -> existingValue,
                        LinkedHashMap::new
                ));

        // 3-a. 최근 1 month 데이터가 없는 경우, 누적 데이터를 응답
        if(검색결과.isEmpty()) {
            요구사항_지라이슈상태_주별_집계 defaultAggregation = new 요구사항_지라이슈상태_주별_집계();
            defaultAggregation.setTotalIssues(totalIssues);
            defaultAggregation.setTotalRequirements(totalRequirements);
            defaultAggregation.setStatuses(new HashMap<>(totalStatuses));
            검색결과.put("최근 한 달 간의 데이터가 존재하지 않아 누적 데이터를 표시합니다.", defaultAggregation);
        } else {
            // 3-b. 검색 데이터에 누적 데이터를 더해준다.
            for(요구사항_지라이슈상태_주별_집계 monthlyAggregation : 검색결과.values()) {
                totalIssues += monthlyAggregation.getTotalIssues();
                totalRequirements += monthlyAggregation.getTotalRequirements();
                monthlyAggregation.setTotalIssues(totalIssues);
                monthlyAggregation.setTotalRequirements(totalRequirements);

                Map<String, Long> currentStatuses = monthlyAggregation.getStatuses();
                for(Map.Entry<String, Long> entry : currentStatuses.entrySet()) {
                    totalStatuses.merge(entry.getKey(), entry.getValue(), Long::sum);
                }

                monthlyAggregation.setStatuses(new HashMap<>(totalStatuses));
            }
        }

        return 검색결과;
    }

    private 요구사항_지라이슈상태_주별_집계 누적데이터조회(Long pdServiceLink, List<Long> pdServiceVersionLinks, LocalDate monthAgo) {
        // 총 이슈 개수를 구하기 위한 쿼리
        EsQuery issueEsQuery = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", pdServiceLink),
                        new TermsQueryFilter("pdServiceVersions", pdServiceVersionLinks),
                        RangeQueryFilter.of("created").lt(monthAgo)
                );
        기본_검색_집계_요청 일반_집계_요청 = new 기본_검색_집계_요청(){};
        일반_집계_요청.set메인_그룹_필드("status.status_name.keyword");
        버킷_집계_결과_목록_합계 searchResponseForTotalIssues = 지라이슈_저장소.aggregateRecentDocs(기본_쿼리_생성기.집계검색(일반_집계_요청,issueEsQuery).생성());
        Long totalIssuesCount = searchResponseForTotalIssues.get전체합계();

        // 총 요구사항 개수를 구하기 위한 쿼리
        EsQuery reqEsQuery = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", pdServiceLink),
                        new TermQueryMust("isReq", true),
                        RangeQueryFilter.of("created").lt(monthAgo)
                );

        Long totalRequirementsCount = Long.valueOf(
                Optional.ofNullable(지라이슈_저장소.findRecentHits(기본_쿼리_생성기.기본검색(reqEsQuery).생성()).toDocs())
                        .map(List::size)
                        .orElse(0)
        );

        // 이슈 상태 집계 결과 가져오기
        Map<String, Long> statusMap = new HashMap<>();
        Optional.of(searchResponseForTotalIssues)
                .map(response -> response.get검색결과().get("group_by_status.status_name.keyword"))
                .ifPresent(totalStatus -> {
                    for (버킷_집계_결과 버킷_집계_결과 : totalStatus) {
                        String key = 버킷_집계_결과.get필드명();
                        long value = 버킷_집계_결과.get개수();
                        statusMap.put(key, value);
                    }
                });

        return new 요구사항_지라이슈상태_주별_집계(totalIssuesCount, statusMap, totalRequirementsCount);
    }

    private 요구사항_지라이슈상태_주별_집계 주별데이터생성(버킷_집계_결과 검색_결과) {

        Map<String, Long> statuses = (검색_결과.get하위검색결과().get("group_by_status.status_name.keyword")).stream()
                .collect(Collectors.toMap(버킷_집계_결과::get필드명, 버킷_집계_결과::get개수));

        long totalReqCount = Optional.ofNullable(검색_결과.get하위검색결과().get("isReq"))
                .flatMap(reqs -> reqs.stream()
                        .filter(bucket -> "true".equals(bucket.get필드명()))
                        .findFirst())
                .map(버킷_집계_결과::get개수)
                .orElse(0L);

        return new 요구사항_지라이슈상태_주별_집계(검색_결과.get개수(), statuses, totalReqCount);
    }

    private String transformDate(String date) {
        OffsetDateTime offsetDateTime = OffsetDateTime.parse(date);
        return DateTimeFormatter
                .ofPattern("yyyy-MM-dd")
                .format(offsetDateTime);
    }

    @Override
    public 버킷_집계_결과_목록_합계 ALM_이슈_일별_제품_버전_집계_조회(ALM_이슈_일별_제품_및_버전_집계_요청 ALM_이슈_일별_제품_및_버전_집계_요청) {
        Long 제품_아이디 = ALM_이슈_일별_제품_및_버전_집계_요청.getPdServiceLink();
        List<Long> 버전_배열 = ALM_이슈_일별_제품_및_버전_집계_요청.getPdServiceVersionLinks();
        Boolean 요구사항_이슈_True = isReqOrGetNull(ALM_이슈_일별_제품_및_버전_집계_요청.getIsReqType());

        String 기준일_필드명 = ALM_이슈_일별_제품_및_버전_집계_요청.get기준일_필드명();
        String 시작일 = ALM_이슈_일별_제품_및_버전_집계_요청.get시작일();
        String 종료일 = ALM_이슈_일별_제품_및_버전_집계_요청.get종료일();

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", 제품_아이디),
                        new TermsQueryFilter("pdServiceVersions", 버전_배열),
                        new TermQueryMust("isReq", 요구사항_이슈_True),
                        RangeQueryFilter.of(기준일_필드명).betweenDate(시작일,종료일)
                );

        기본_검색_집계_하위_요청 하위_집계_요청 = new 기본_검색_집계_하위_요청(){};
        하위_집계_요청.set메인_그룹_필드(기준일_필드명);

        List<집계_하위_필드> 집계_하위_필드들 = new ArrayList<>();

        Optional.ofNullable(ALM_이슈_일별_제품_및_버전_집계_요청.get메인_그룹_필드())
                .ifPresent(element->{
                    집계_하위_필드들.add(
                        집계_하위_필드.builder()
                            .하위_필드명(element)
                            .build());
                    ALM_이슈_일별_제품_및_버전_집계_요청.get집계_하위_필드들()
                            .stream().findFirst()
                            .ifPresent(집계_하위_필드들::add);
                });

        하위_집계_요청.set집계_하위_필드들(집계_하위_필드들);

        return 지라이슈_저장소.aggregateRecentDocs(중첩_집계_쿼리_생성기.포괄_일별(하위_집계_요청,esQuery).생성());

    }

    @Override
    public List<AlmIssueEntity> 지라이슈_기준일자별_제품_및_제품버전_업데이트된_이슈조회(ALM_이슈_일별_제품_및_버전_집계_요청 ALM_이슈_일별_제품_및_버전_집계_요청){

        String 시작일 = ALM_이슈_일별_제품_및_버전_집계_요청.get시작일();
        String 종료일 = ALM_이슈_일별_제품_및_버전_집계_요청.get종료일();

        String from = 시작일;
        String to = 종료일;

        EsBoolQuery[] esBoolQueries = Stream.of(
                new TermQueryMust("pdServiceId", ALM_이슈_일별_제품_및_버전_집계_요청.getPdServiceLink()),
                new TermsQueryFilter("pdServiceVersions", ALM_이슈_일별_제품_및_버전_집계_요청.getPdServiceVersionLinks()),
                new TermQueryMust("isReq", isReqOrGetNull(ALM_이슈_일별_제품_및_버전_집계_요청.getIsReqType())),
                new RangeQueryFilter(ALM_이슈_일별_제품_및_버전_집계_요청.get기준일_필드명(), from, to, "fromto")
        ).filter(Objects::nonNull).toArray(EsBoolQuery[]::new);

        EsQueryBuilder esQuery
                = new EsQueryBuilder()
                .bool(esBoolQueries)
                .sort(new EsSortQuery(
                        List.of(
                                기본_정렬_요청.builder().필드(ALM_이슈_일별_제품_및_버전_집계_요청.get기준일_필드명()).정렬_기준("asc").build()
                        )
                ));

        return 지라이슈_저장소.findRecentHits(기본_쿼리_생성기.기본검색(esQuery).생성()).toDocs();
    }

    @Override
    public 버킷_집계_결과_목록_합계 현황관리_연결이슈_하위이슈_집계(ALM_이슈_제품_및_버전_집계_요청 종합_집계_요청, Long 요구사항_링크) {
        Boolean isReq = Optional.ofNullable(종합_집계_요청.getIsReqType())
                .map(IsReqType::name)
                .map(name -> name.equals(REQUIREMENT.name()) ? Boolean.TRUE
                        : name.equals(ISSUE.name()) ? Boolean.FALSE
                        : null)
                .orElse(null);

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", 종합_집계_요청.getPdServiceLink()),
                        new TermQueryMust("isReq", isReq),
                        new TermsQueryFilter("pdServiceVersions", 종합_집계_요청.getPdServiceVersionLinks()),
                        new TermQueryMust("cReqLink", 요구사항_링크)
                );


        버킷_집계_결과_목록_합계 집계_결과 = 지라이슈_저장소.aggregateRecentDocs(중첩_집계_쿼리_생성기.포괄(종합_집계_요청, esQuery).생성());
        return 집계_결과;
    }

    @Override
    public Map<String,List<요구사항_지라이슈키별_업데이트_목록_데이터>> 요구사항_지라이슈키별_업데이트_목록(List<String> 지라키_목록){

        String[] names = new String[]{"parentReqKey","key"};
        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        지라키_목록.stream().flatMap(지라키 -> Arrays.stream(names)
                                .map(name -> new TermsQueryFilter(name, 지라키))
                                .collect(toList()).stream()).toArray(TermsQueryFilter[]::new)
                );


        List<AlmIssueEntity> 전체결과 = new ArrayList<>();

        boolean 인덱스존재시까지  = true;
        LocalDate today = LocalDate.now();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        String 지라인덱스 = "jiraissue";

        while(인덱스존재시까지) {
            LocalDate 오늘일경우 = LocalDate.now();
            String 호출할_지라인덱스 = 오늘일경우.format(formatter).equals(today.format(formatter))
                    ? 지라인덱스 : 지라인덱스 + "-" + today.format(formatter);

            if (!지라이슈_저장소.인덱스_존재_확인(호출할_지라인덱스)) {
                인덱스존재시까지 = false;
                break;
            }

            today = today.minusDays(1);

            List<AlmIssueEntity> 결과 = 지라이슈_저장소.findHits(
                    기본_쿼리_생성기.기본검색(esQuery).생성(), 호출할_지라인덱스).toDocs();

            if (결과 != null && 결과.size() > 0) {
                전체결과.addAll(결과);
            }
        }

        Map<String, List<요구사항_지라이슈키별_업데이트_목록_데이터>> 조회_결과 = 전체결과.stream()
                .map(this::요구사항_지라이슈키별_업데이트_목록_데이터)
                .distinct()
                .collect(Collectors.groupingBy(data -> data.getIsReq() ? data.getKey() : data.getParentReqKey()));

        // 하위 이슈로 작업한 경우 요구사항 데이터는 제거
        조회_결과.forEach((key, valueList) -> {
            if (valueList.stream().anyMatch(data -> data.getIsReq() == false)) {
                valueList.removeIf(data -> data.getIsReq() == true);
            }
        });

        return 조회_결과;
    }

    private 요구사항_지라이슈키별_업데이트_목록_데이터 요구사항_지라이슈키별_업데이트_목록_데이터(AlmIssueEntity AlmIssueEntity){
        return new 요구사항_지라이슈키별_업데이트_목록_데이터(
                AlmIssueEntity.getKey(),
                AlmIssueEntity.getParentReqKey(),
                AlmIssueEntity.getUpdated(),
                AlmIssueEntity.getResolutiondate(),
                AlmIssueEntity.getIsReq()
        );

    }

}
