package com.arms.api.analysis.resource.controller;

import com.arms.api.analysis.resource.dto.ResourceAggrWithAssigneesSubGroupDTO;
import com.arms.api.analysis.resource.dto.ResourceDTO;
import com.arms.api.analysis.resource.dto.ResourceWithVersionIdNamesDTO;
import com.arms.api.analysis.resource.service.AnalysisResource;
import com.arms.api.analysis.resource.vo.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/engine/analysis-resource")
@Slf4j
@AllArgsConstructor
public class AnalysisResourceController {

    private final AnalysisResource analysisResource;

    @PostMapping("/word-cloud")
    public ResponseEntity<List<WordCloudVO>> findWordCloudAggregationData(@RequestBody ResourceDTO resourceDTO) {

        log.info("AnalysisResourceController :: findWordCloudAggregationData");

        return ResponseEntity.ok(analysisResource.findWordCloudAggregationData(resourceDTO));
    }

    // worker-status datatable 용도
    @PostMapping("/issue-status-by-assignee")
    public ResponseEntity<List<UniqueAssigneeIssueStatusVO>> issueStatusByAssignee(@RequestBody ResourceDTO resourceDTO) {
        return ResponseEntity.ok(analysisResource.issueStatusByAssignee(resourceDTO));
    }

    @PostMapping("/issue-status-stacked-bar")
    public ResponseEntity<StackedHorizontalBarChartVO> issueStatusStackedBarChartAggregationData(@RequestBody ResourceDTO resourceDTO) {

        log.info("AnalysisResourceController :: issueStatusStackedBarChartAggregationData");

        return ResponseEntity.ok(analysisResource.issueStatusStackedBarChartAggregationData(resourceDTO));
    }

    // 작업자_이슈_상위_N명_집계
    @PostMapping("/top-contributors")
    public ResponseEntity<TotalIssueAndPieChartVO> findTopNContributorsPieChartAggregationData(@RequestBody ResourceDTO resourceDTO) {

        log.info("AnalysisResourceController :: findTopNContributorsPieChartAggregationData");

        return ResponseEntity.ok(analysisResource.findTopNContributorsPieChartAggregationData(resourceDTO));
    }

    @PostMapping("/top-contributors/issue-properties")
    public ResponseEntity<HorizontalBarChartYAxisAndSeriesVO> findTopNContributorsHorizontalBarChartAggregationData(
            @RequestBody ResourceAggrWithAssigneesSubGroupDTO resourceAggrWithAssigneesSubGroupDTO) {

        log.info("AnalysisResourceController :: findTopNContributorsHorizontalBarChartAggregationData");

        return ResponseEntity.ok(analysisResource.findTopNContributorsHorizontalBarChartAggregationData(resourceAggrWithAssigneesSubGroupDTO));
    }

    // Sankey
    @PostMapping(value="/version-assignees", name="resource-findVersionAssignees")
    public ResponseEntity<List<AssigneePerVersionVO>> findVersionAssignees(@RequestBody ResourceDTO resourceDTO) {

        log.info("AnalysisResourceController :: findVersionAssignees");

        return ResponseEntity.ok(analysisResource.findVersionAssignees((resourceDTO)));
    }

    // TreeMap
    @PostMapping(value="/assignees-requirements-involvements", name="resource-treemap")
    public ResponseEntity<List<TreeMapWorkerVO>> 작업자_별_요구사항_별_관여도_treemap(@RequestBody ResourceWithVersionIdNamesDTO resourceWithVersionIdNamesDTO) {

        log.info("[AnalysisResourceController :: 작업자_별_요구사항_별_관여도_treemap]");

        return ResponseEntity.ok(analysisResource.작업자_별_요구사항_별_관여도_treemap(resourceWithVersionIdNamesDTO));

    }

    // normal-version-only/{pdServiceId} 대체
    @PostMapping("/assignees-info")
    public ResponseEntity<List<UniqueAssigneeVO>> findAssigneesInfo(@RequestBody ResourceDTO resourceDTO) {
        log.info("AnalysisResourceController :: findAssigneesInfo");

        return ResponseEntity.ok(analysisResource.findAssigneesInfo(resourceDTO));
    }

}
