package com.arms.api.analysis.resource.service;

import com.arms.api.analysis.resource.dto.*;
import com.arms.api.analysis.resource.vo.*;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.util.model.dto.PdServiceAndIsReqDTO;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.ExistsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.model.dto.집계_하위_필드;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentBucket;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

@Slf4j
@Service("analysisResource")
@AllArgsConstructor
public class AnalysisResourceImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements AnalysisResource {

    @Override
    public List<AssigneePerVersionVO> findVersionAssignees(ResourceDTO resourceDTO) {

        ResourceAggrDTO resourceAggrDTO = new ResourceAggrDTO();
        resourceAggrDTO.setPdServiceAndIsReq(resourceDTO.getPdServiceAndIsReq());

        EsQuery notReqIssueQuery = esQueryWithAssignee(resourceAggrDTO, Boolean.FALSE);

        resourceAggrDTO.set메인_그룹_필드("pdServiceVersions");
        resourceAggrDTO.set메인_그룹_필드명_별칭("pdServiceVersions");
        resourceAggrDTO.set집계_하위_필드들(
            List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeID")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .크기(5) // 버전별 5명씩
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeName")
                    .하위_필드명("assignee.assignee_displayName.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeEmail")
                    .하위_필드명("assignee.assignee_emailAddress.keyword")
                    .build()
            )
        );
        resourceAggrDTO.setEsQuery(notReqIssueQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(resourceAggrDTO);

        Map<Long, AssigneePerVersionVO> versionIdVOMap = new HashMap<>();

        for (DocumentBucket documentBucket : documentAggregations.deepestSubList()) {

            Long versionId = Long.valueOf(documentBucket.getDocFieldName("pdServiceVersions"));
            AssigneeIssueCountVO assigneeIssueCountVO = AssigneeIssueCountVO
                .builder()
                .name(documentBucket.getDocFieldName("assigneeName"))
                .accountId(documentBucket.getDocFieldName("assigneeID"))
                .emailAddress(Optional.ofNullable(documentBucket.getDocFieldName("assigneeEmail")).orElse(""))
                .issueCount(documentBucket.getDocCount())
                .build();

            if (versionIdVOMap.containsKey(versionId)) {
                AssigneePerVersionVO assigneePerVersionVO = versionIdVOMap.get(versionId);
                List<AssigneeIssueCountVO> assigneeIssueCounts = assigneePerVersionVO.getAssigneeIssueCounts();
                assigneeIssueCounts.add(assigneeIssueCountVO);
            } else {

                AssigneePerVersionVO assigneePerVersionVO = AssigneePerVersionVO.builder()
                    .versionId(versionId)
                    .assigneeIssueCounts(new ArrayList<>(List.of(assigneeIssueCountVO)))
                    .build();
                versionIdVOMap.put(versionId, assigneePerVersionVO);
            }
        }

        return new ArrayList<>(versionIdVOMap.values());
    }

    @Override
    public List<UniqueAssigneeIssueStatusVO> issueStatusByAssignee(ResourceDTO resourceDTO) {

        ResourceAggrDTO resourceAggrDTO = new ResourceAggrDTO();
        resourceAggrDTO.setPdServiceAndIsReq(resourceDTO.getPdServiceAndIsReq());

        resourceAggrDTO.set메인_그룹_필드("jira_server_id");
        resourceAggrDTO.set집계_하위_필드들(List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeID")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("statusName")
                    .하위_필드명("status.status_name.keyword")
                    .build()
            )
        );
        resourceAggrDTO.set크기(10000);
        resourceAggrDTO.set하위_크기(10000);

        EsQuery reqIssueQuery
            = esQueryWithAssignee(resourceAggrDTO, Boolean.TRUE);
        EsQuery notReqIssueQuery
            = esQueryWithAssignee(resourceAggrDTO, Boolean.FALSE);

        // 0. req & nonReq 인 사용자 정보 Map을 만든다.
        // 유일한 AssigneeMap (uniqueId :: serverId + "-" + accountId)
        Map<String, UniqueAssigneeVO> uniqueAssigneeMap
            = mergeMaps(uniqueAssigneeMap(reqIssueQuery, resourceAggrDTO), uniqueAssigneeMap(notReqIssueQuery, resourceAggrDTO));

        // 실제 요구사항이슈 집계 내용
        resourceAggrDTO.setEsQuery(reqIssueQuery);
        DocumentAggregations reqIssueAggregations = this.aggregateRecentDocs(resourceAggrDTO);
        // 하위 및 연결이슈 집계 내용
        resourceAggrDTO.setEsQuery(notReqIssueQuery);
        DocumentAggregations notReqIssueAggregations = this.aggregateRecentDocs(resourceAggrDTO);

        // 1.1 요구사항 상태 집계 데이터 활용해서 (req) UniqueId 및 List<IssuePropertyVO> 를 도출한다.
        Map<String, List<IssuePropertyVO>> reqIssueUniqueIdStatusData = uniqueIdStatusName(reqIssueAggregations);
        // 1.2 요구사항 상태 집계 데이터 활용해서 (nonReq) UniqueId 및 List<IssuePropertyVO> 를 도출한다.
        Map<String, List<IssuePropertyVO>> nonReqIssueUniqueIdStatusData = uniqueIdStatusName(notReqIssueAggregations);

        // 2.1 1과 0을 활용하여 => UniqueAssigneeIssueStatusVO (req) & UniqueAssigneeIssueStatusVO (nonReq) 생성
        // 2.2 List<UniqueAssigneeIssueStatusVO> 도출
        List<UniqueAssigneeIssueStatusVO> issueStatusData = buildUniqueAssigneeIssueStatusList(uniqueAssigneeMap, reqIssueUniqueIdStatusData, nonReqIssueUniqueIdStatusData);

        //3. TotalIssueCount, reqIssueCount, notReqIssueCount 순으로 정렬후 반환
        return sortByTotalReqNotReqCountsDesc(issueStatusData);
    }

    private List<UniqueAssigneeIssueStatusVO> sortByTotalReqNotReqCountsDesc(List<UniqueAssigneeIssueStatusVO> issueStatusData) {
        issueStatusData.sort(Comparator
            .comparingLong(UniqueAssigneeIssueStatusVO::getTotalIssueCount).reversed()
            .thenComparingLong(UniqueAssigneeIssueStatusVO::getReqIssueCount).reversed()
            .thenComparingLong(UniqueAssigneeIssueStatusVO::getNotReqIssueCount).reversed()
        );
        return issueStatusData;
    }

    private List<UniqueAssigneeIssueStatusVO> buildUniqueAssigneeIssueStatusList (
        Map<String, UniqueAssigneeVO> uniqueAssigneeMap,
        Map<String, List<IssuePropertyVO>> reqIssueUniqueIdStatusData,
        Map<String, List<IssuePropertyVO>> nonReqIssueUniqueIdStatusData
    ) {
        List<UniqueAssigneeIssueStatusVO> returnVO = new ArrayList<>();

        for (Map.Entry<String, UniqueAssigneeVO> entry : uniqueAssigneeMap.entrySet()) {
            String key = entry.getKey();

            UniqueAssigneeVO uniqueAssigneeVO = entry.getValue();

            List<IssuePropertyVO> reqIssueProperties = reqIssueUniqueIdStatusData.getOrDefault(key, Collections.emptyList());
            List<IssuePropertyVO> notReqIssueProperties = nonReqIssueUniqueIdStatusData.getOrDefault(key, Collections.emptyList());

            Long reqIssueCount = reqIssueProperties.stream()
                .mapToLong(IssuePropertyVO::getValue)
                .sum();
            Long notReqIssueCount = notReqIssueProperties.stream()
                .mapToLong(IssuePropertyVO::getValue)
                .sum();;
            Long totalIssueCount = reqIssueCount + notReqIssueCount;

            UniqueAssigneeIssueStatusVO uniqueAssigneeIssueStatusVO = UniqueAssigneeIssueStatusVO.builder()
                .uniqueAssigneeVO(uniqueAssigneeVO)
                .totalIssueCount(totalIssueCount)
                .reqIssueCount(reqIssueCount)
                .notReqIssueCount(notReqIssueCount)
                .reqIssueProperties(reqIssueProperties)
                .notReqIssueProperties(notReqIssueProperties)
                .build();

            returnVO.add(uniqueAssigneeIssueStatusVO);
        }

        return returnVO;
    }


    private Map<String, List<IssuePropertyVO>> uniqueIdStatusName(DocumentAggregations aggregations) {
        List<DocumentBucket> deepestSubList = aggregations.deepestSubList();

        Map<String, List<IssuePropertyVO>> uniqueIdIssueProperties = new HashMap<>();

        for (DocumentBucket documentBucket : deepestSubList) {
            String serverId = documentBucket.getDocFieldName("group_by_jira_server_id"); // 메인그룹필드 docFieldName
            String accountId = documentBucket.getDocFieldName("assigneeID");

            String uniqueId = serverId + "-" + accountId;

            String statusName = documentBucket.getDocFieldName("statusName");
            Long docCount = documentBucket.getDocCount();

            IssuePropertyVO propertyVO = IssuePropertyVO.builder()
                .name(statusName)
                .value(docCount)
                .build();

            // map에 uniqueId가 있으면, IssuePropertyVo 리스트를 가져와서, propertyVO를 add,
            // 없으면 map에 uniqueId를 key로하고 IssuePropertyVO 가 있는 리스트를 만들어서 넣기.
            if (uniqueIdIssueProperties.containsKey(uniqueId)) {
                List<IssuePropertyVO> issueProperties = uniqueIdIssueProperties.get(uniqueId);
                issueProperties.add(propertyVO);
            } else {
                uniqueIdIssueProperties.put(uniqueId, new ArrayList<>(List.of(propertyVO)));
            }
        }
        return uniqueIdIssueProperties;
    }

    // Map 2개 Merge
    private Map<String, UniqueAssigneeVO> mergeMaps(Map<String, UniqueAssigneeVO> map1, Map<String, UniqueAssigneeVO> map2) {

        Map<String, UniqueAssigneeVO> mergedMap = new HashMap<>(map1);

        mergedMap.putAll(map2);

        return mergedMap;
    }

    private boolean isUniqueKey(String uniqueId, Set<String> uniqueIds) {
        if(!uniqueIds.contains(uniqueId)) {
            return true;
        }
        return false;
    }

    // uniqueId = "{ALM_server_id}-{assignee.assignee_accountId}" 형태
    private Map<String, UniqueAssigneeVO> uniqueAssigneeMap(EsQuery esQuery, ResourceAggrDTO resourceAggrDTO) {
        PdServiceAndIsReqDTO pdServiceAndIsReq = resourceAggrDTO.getPdServiceAndIsReq();

        ResourceAggrDTO aggrDTO = new ResourceAggrDTO();

        aggrDTO.setPdServiceAndIsReq(
            new PdServiceAndIsReqDTO(pdServiceAndIsReq.getPdServiceId(),
                                    pdServiceAndIsReq.getPdServiceVersions(),
                                    pdServiceAndIsReq.getIsReq())
        );
        aggrDTO.set메인_그룹_필드("jira_server_id");
        aggrDTO.set집계_하위_필드들(List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeID")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeName")
                    .하위_필드명("assignee.assignee_displayName.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeEmail")
                    .하위_필드명("assignee.assignee_emailAddress.keyword")
                    .build()
            )
        );
        aggrDTO.setEsQuery(esQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(aggrDTO);
        List<DocumentBucket> deepestSubList = documentAggregations.deepestSubList();

        Set<String> uniqueIds = new HashSet<>();
        Map<String, UniqueAssigneeVO> uniqueAssigneeMap = new HashMap<>();

        for (DocumentBucket documentBucket : deepestSubList) {
            String serverId = documentBucket.getDocFieldName("group_by_jira_server_id"); // 메인그룹필드 docFieldName
            String accountId = documentBucket.getDocFieldName("assigneeID");
            String name = documentBucket.getDocFieldName("assigneeName");
            String assigneeEmail = documentBucket.getDocFieldName("assigneeEmail");
            String uniqueId = serverId + "-" + accountId;

            UniqueAssigneeVO uniqueAssigneeVO = UniqueAssigneeVO.builder()
                .serverId(serverId)
                .accountId(accountId)
                .name(name)
                .emailAddress(Optional.ofNullable(assigneeEmail).orElse(""))
                .uniqueId(uniqueId)
                .build();

            if(isUniqueKey(uniqueId, uniqueIds)) {
                uniqueIds.add(uniqueId);
                uniqueAssigneeMap.put(uniqueId, uniqueAssigneeVO);
            }

        }

        return uniqueAssigneeMap;
    }

    private EsQuery esQueryWithAssignee(ResourceAggrDTO resourceAggrDTO, Boolean isReq) {
        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                new TermQueryMust("pdServiceId",resourceAggrDTO.getPdServiceLink()),
                new TermsQueryFilter("pdServiceVersions",resourceAggrDTO.getPdServiceVersionLinks()),
                new TermQueryMust("isReq", isReq),
                new ExistsQueryFilter("assignee")
            );
        return esQuery;
    }

    // word-cloud chart
    @Override
    public List<WordCloudVO> findWordCloudAggregationData(ResourceDTO resourceDTO) {

        ResourceAggrDTO resourceAggrDTO = new ResourceAggrDTO();
        resourceAggrDTO.setPdServiceAndIsReq(resourceDTO.getPdServiceAndIsReq());

        resourceAggrDTO.set메인_그룹_필드("jira_server_id");
        resourceAggrDTO.set메인_그룹_필드명_별칭("serverId");
        resourceAggrDTO.set집계_하위_필드들(List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeID")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeName")
                    .하위_필드명("assignee.assignee_displayName.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeEmail")
                    .하위_필드명("assignee.assignee_emailAddress.keyword")
                    .build()
            )
        );
        PdServiceAndIsReqDTO pdServiceAndIsReq = resourceAggrDTO.getPdServiceAndIsReq();
        pdServiceAndIsReq.setIsReq(false); // 요구사항_이슈가 아닌 이슈 (하위이슈, 연결이슈 대상)
        resourceAggrDTO.set크기(10000);
        resourceAggrDTO.set하위_크기(10000);

        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                new TermQueryMust("pdServiceId", resourceAggrDTO.getPdServiceLink()),
                new TermsQueryFilter("pdServiceVersions", resourceAggrDTO.getPdServiceVersionLinks()),
                new TermQueryMust("isReq", resourceAggrDTO.isReq()),
                new ExistsQueryFilter("assignee")
            );

        resourceAggrDTO.setEsQuery(esQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(resourceAggrDTO);

        return toWordCloudVO(documentAggregations);
    }

    // toListOfWorkerIssueCountFromResultOfAggregation
    private List<WordCloudVO> toWordCloudVO(DocumentAggregations documentAggregations) {
        List<WordCloudVO> wordClouds = new ArrayList<>();

        List<DocumentBucket> documentBuckets = documentAggregations.deepestSubList();

        for (DocumentBucket documentBucket : documentBuckets) {
            String assigneeName = documentBucket.getDocFieldName("assigneeName");
            String assigneeEmail = documentBucket.getDocFieldName("assigneeEmail");

            WordCloudVO.WordCloudVOBuilder wordCloudVOBuilder = WordCloudVO.builder();

            if (assigneeEmail != null && !assigneeEmail.isEmpty()) {
                String emailId = getEmailId(assigneeEmail);
                wordCloudVOBuilder.text(assigneeName + Optional.ofNullable(emailId).map(id -> " ("+id+")").orElse(""));
            } else {
                wordCloudVOBuilder.text(assigneeName);
            }
            wordCloudVOBuilder.weight(documentBucket.getDocCount());

            wordClouds.add(wordCloudVOBuilder.build());
        }

        return wordClouds;
    }

    @Override
    public StackedHorizontalBarChartVO issueStatusStackedBarChartAggregationData(ResourceDTO resourceDTO) {

        ResourceAggrDTO resourceAggrDTO = new ResourceAggrDTO();
        resourceAggrDTO.setPdServiceAndIsReq(resourceDTO.getPdServiceAndIsReq());

        resourceAggrDTO.set메인_그룹_필드("jira_server_id");
        resourceAggrDTO.set집계_하위_필드들(
            List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeID")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("statusName")
                    .하위_필드명("status.status_name.keyword")
                    .build()
            )
        );

        // 1. 작업자 정보 세팅 (uniqueAssigneeMap)
        EsQuery notReqIssueQuery
            = esQueryWithAssignee(resourceAggrDTO, Boolean.FALSE);
        Map<String, UniqueAssigneeVO> uniqueAssigneeMap = uniqueAssigneeMap(notReqIssueQuery, resourceAggrDTO);
        resourceAggrDTO.setEsQuery(notReqIssueQuery);
        // 2. 집계 결과 확인.
        DocumentAggregations notReqIssueAggregations =
            this.aggregateRecentDocs(resourceAggrDTO);

        List<DocumentBucket> deepestSubList = notReqIssueAggregations.deepestSubList();
        if(deepestSubList.isEmpty()) {
            log.info("issueStatusStackedBarChartAggregationData :: deepestSubList.size => 0");
            throw new RuntimeException("issueStatusStackedBarChartAggregationData :: deepestSubList.size => 0");
        }

        // unique - IssuePropertyies MAP 생성
        Map<String, List<IssuePropertyVO>> uniqueIdIssueProperties = uniqueIdIssueProperties(deepestSubList);

        // 시리즈의 기초 names - Make Set firtst and convert to List
        List<String> statusNames = new ArrayList<>(statusNameSet(deepestSubList));

        // uniqueIdIssueProperties 에서 yAxis에 넣을 이름순서 설정
        List<UniqueAssigneeVO> assignees = assigneesByUniqueIdIssueProperties(uniqueAssigneeMap, uniqueIdIssueProperties);
        List<String> assigneeNames = assignees.stream()
            .map(UniqueAssigneeVO::getName)
            .collect(Collectors.toList());
        // yAxisVO 설정 완료
        StackedHorizontalBarChartYAxisVO chartYAxisVO = StackedHorizontalBarChartYAxisVO.builder()
            .type("category")
            .data(assigneeNames)
            .build();

        // 4. Series 만들기 (by issueStatus, assignees, assignee-properties )
        List<StackedHorizontalBarSeriesVO> series = stackedHorizontalBarSeriesVO(statusNames, assignees, uniqueIdIssueProperties);

        return StackedHorizontalBarChartVO.builder()
            .yAxisVO(chartYAxisVO)
            .series(series)
            .build();
    }

    private Set<String> statusNameSet(List<DocumentBucket> deepestSubList) {
        Set<String> statusNameSet = new HashSet<>();
        for (DocumentBucket documentBucket : deepestSubList) {
            String statusName = documentBucket.getDocFieldName("statusName");
            Long docCount = documentBucket.getDocCount();

            statusNameSet.add(statusName); // statusName을 넣기.
        }
        return statusNameSet;
    }

    private Map<String, List<IssuePropertyVO>> uniqueIdIssueProperties(List<DocumentBucket> deepestSubList) {

        Map<String, List<IssuePropertyVO>> uniqueIdIssueProperties = new HashMap<>();

        for (DocumentBucket documentBucket : deepestSubList) {
            String serverId = documentBucket.getDocFieldName("group_by_jira_server_id"); // 메인그룹필드 docFieldName
            String accountId = documentBucket.getDocFieldName("assigneeID");

            String uniqueId = serverId + "-" + accountId;

            String statusName = documentBucket.getDocFieldName("statusName");
            Long docCount = documentBucket.getDocCount();

            IssuePropertyVO propertyVO = IssuePropertyVO.builder()
                .name(statusName)
                .value(docCount)
                .build();

            // map에 uniqueId가 있으면, IssuePropertyVO 리스트를 가져와서, propertyVO를 add,
            // 없으면 map에 uniqueId를 key로하고 IssuePropertyVO 가 있는 리스트를 만들어서 넣기.
            if (uniqueIdIssueProperties.containsKey(uniqueId)) {
                List<IssuePropertyVO> issueProperties = uniqueIdIssueProperties.get(uniqueId);
                issueProperties.add(propertyVO);
            } else {
                uniqueIdIssueProperties.put(uniqueId, new ArrayList<>(List.of(propertyVO)));
            }
        }
        return uniqueIdIssueProperties;
    }

    private List<UniqueAssigneeVO> assigneesByUniqueIdIssueProperties(Map<String, UniqueAssigneeVO> uniqueAssigneeMap, Map<String, List<IssuePropertyVO>> uniqueIdIssueProperties) {
        List<UniqueAssigneeVO> assignees = new ArrayList<>();

        // uniqueId에 대한, 이름 (이메일아이디) 에 대한 세팅으로 HorizontalStackedBarChartYAxisVO
        for (Map.Entry<String, List<IssuePropertyVO>> entry : uniqueIdIssueProperties.entrySet()) {
            String uniqueId = entry.getKey();

            UniqueAssigneeVO uniqueAssigneeVO = uniqueAssigneeMap.get(uniqueId);

            String nameWithEmailId = uniqueAssigneeVO.getName();
            if (!uniqueAssigneeVO.getEmailAddress().isEmpty()) {
                String emailId = getEmailId(uniqueAssigneeVO.getEmailAddress());
                nameWithEmailId += " (" + emailId +")";
            }

            UniqueAssigneeVO vo = UniqueAssigneeVO.builder()
                .uniqueId(uniqueId)
                .name(nameWithEmailId)
                .serverId(uniqueAssigneeVO.getServerId())
                .emailAddress(uniqueAssigneeVO.getEmailAddress())
                .build();

            assignees.add(vo);
        }
        return assignees;
    }

    private List<StackedHorizontalBarSeriesVO> stackedHorizontalBarSeriesVO(List<String> statusNames, List<UniqueAssigneeVO> assignees, Map<String, List<IssuePropertyVO>> uniqueIdIssueProperties) {
        List<StackedHorizontalBarSeriesVO> series = new ArrayList<>();

        for (String statusName : statusNames) {
            List<Long> data = new ArrayList<>();

            for (UniqueAssigneeVO assignee : assignees) {
                List<IssuePropertyVO> issueProperties = uniqueIdIssueProperties.get(assignee.getUniqueId());
                Optional<IssuePropertyVO> issueStatusProperty = issueProperties.stream()
                    .filter(proterty -> proterty.getName().equals(statusName))
                    .findFirst();

                if (issueStatusProperty.isPresent()) {
                    data.add(issueStatusProperty.get().getValue());
                } else {
                    data.add(0L);
                }
            }
            series.add(
                StackedHorizontalBarSeriesVO.builder()
                    .type("bar")
                    .name(statusName)
                    .data(data).build()
            );
        }
        return series;
    }

    @Override
    public TotalIssueAndPieChartVO findTopNContributorsPieChartAggregationData(ResourceDTO resourceDTO) {

        ResourceAggrDTO resourceAggrDTO = new ResourceAggrDTO();
        resourceAggrDTO.setPdServiceAndIsReq(resourceDTO.getPdServiceAndIsReq());

        resourceAggrDTO.set메인_그룹_필드("jira_server_id");
        resourceAggrDTO.set집계_하위_필드들(
            List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeID")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .크기(resourceDTO.getSize())
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeName")
                    .하위_필드명("assignee.assignee_displayName.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeEmail")
                    .하위_필드명("assignee.assignee_emailAddress.keyword")
                    .build()
            )
        );

        EsQuery esQuery
            = new EsQueryBuilder()
            .bool(
                new TermQueryMust("pdServiceId",resourceAggrDTO.getPdServiceLink()),
                new TermsQueryFilter("pdServiceVersions", resourceAggrDTO.getPdServiceVersionLinks()),
                new TermQueryMust("isReq", resourceAggrDTO.isReq()), // REQUIREMENT or ISSUE
                new ExistsQueryFilter("assignee")
            );

        resourceAggrDTO.setEsQuery(esQuery);
        DocumentAggregations documentAggregations = this.aggregateRecentDocs(resourceAggrDTO);

        return toTotalIssueAndPieChartVO(documentAggregations);

    }

    private TotalIssueAndPieChartVO toTotalIssueAndPieChartVO(DocumentAggregations documentAggregations) {

        List<PieChartVO> assigneeAndIssueCounts = new ArrayList<>();

        documentAggregations.getDocSumCount();

        List<DocumentBucket> documentBuckets = documentAggregations.deepestSubList();

        for (DocumentBucket documentBucket : documentBuckets) {
            String serverId = documentBucket.getDocFieldName("group_by_jira_server_id"); // 메인그룹필드 docFieldName
            String assigneeID = documentBucket.getDocFieldName("assigneeID");
            String assigneeName = documentBucket.getDocFieldName("assigneeName");
            String assigneeEmail = documentBucket.getDocFieldName("assigneeEmail");

            PieChartVO.PieChartVOBuilder pieChartVOBuilder = PieChartVO.builder();
            if (assigneeEmail != null & !assigneeEmail.isEmpty()) {
                pieChartVOBuilder.emailAddress(assigneeEmail);
            }
            PieChartVO pieChartVO = pieChartVOBuilder.serverId(serverId).accountId(assigneeID).name(assigneeName).value(documentBucket.getDocCount()).build();
            assigneeAndIssueCounts.add(pieChartVO);
        }

        TotalIssueAndPieChartVO.TotalIssueAndPieChartVOBuilder totalIssueAndPieChartVOBuilder = TotalIssueAndPieChartVO.builder();

        totalIssueAndPieChartVOBuilder.totalIssueCount(documentAggregations.getTotalHits()); // 전체 hit 수
        totalIssueAndPieChartVOBuilder.assigneeAndIssueCounts(assigneeAndIssueCounts);

        return totalIssueAndPieChartVOBuilder.build();
    }

    private String getEmailId(String emailAddress) {
        // Email Regular Expression
        String EMAIL_REGEX = "^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\\.[A-Za-z]{2,}$";

        // EmailAddress Validation
        if (emailAddress == null || !Pattern.matches(EMAIL_REGEX, emailAddress)) {
            return null; // invalid then return null
        }

        int atIndex = emailAddress.indexOf('@');
        if (atIndex > 0) {
            return emailAddress.substring(0, atIndex);
        }

        return null;
    }

    @Override
    public HorizontalBarChartYAxisAndSeriesVO findTopNContributorsHorizontalBarChartAggregationData(
        ResourceAggrWithAssigneesSubGroupDTO resourceAggrWithAssigneesSubGroupDTO) {
        PdServiceAndIsReqDTO pdServiceAndIsReqDto = resourceAggrWithAssigneesSubGroupDTO.getPdServiceAndIsReq();

        String subGroupField = resourceAggrWithAssigneesSubGroupDTO.getSubGroup();

        resourceAggrWithAssigneesSubGroupDTO.set메인_그룹_필드("jira_server_id");
        resourceAggrWithAssigneesSubGroupDTO.set집계_하위_필드들(
            List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("assigneeID")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .build(),
                집계_하위_필드.builder()
                    .하위_필드명_별칭("propertyName")
                    .하위_필드명(subGroupField)
                    .build()
            )
        );

        List<AssigneeDTO> assigneeDTOs = resourceAggrWithAssigneesSubGroupDTO.getAssignees();

        List<AssigneeIssuePropertiesVO> assigneeIssuePropertiesVOList = new ArrayList<>();
        for (AssigneeDTO assigneeDTO : assigneeDTOs) {
            EsQuery esQuery
                = new EsQueryBuilder()
                .bool(
                    new TermQueryMust("pdServiceId",pdServiceAndIsReqDto.getPdServiceId()),
                    new TermsQueryFilter("pdServiceVersions", pdServiceAndIsReqDto.getPdServiceVersions()),
                    new TermQueryMust("isReq", pdServiceAndIsReqDto.getIsReq()), // REQUIREMENT(true) or ISSUE(false) or null(don't use this condition)
                    new TermQueryMust("jira_server_id",assigneeDTO.getServerId()),
                    new TermQueryMust("assignee.assignee_accountId.keyword",assigneeDTO.getAccountId()),
                    new ExistsQueryFilter("assignee")
                );

            resourceAggrWithAssigneesSubGroupDTO.setEsQuery(esQuery);

            DocumentAggregations documentAggregations =
                this.aggregateRecentDocs(resourceAggrWithAssigneesSubGroupDTO);
            // 1. AssigneeIssuePropertyVOs 로 변환 :: 담당자 정보 + List<IssuePropertyVO>
            AssigneeIssuePropertiesVO assigneeIssuePropertiesVO = toAssigneeIssueProperties(documentAggregations, assigneeDTO);

            assigneeIssuePropertiesVOList.add(assigneeIssuePropertiesVO);
        }

        // 2. HorizontalBarChartYAxisVO => Chart의 YAxios VO
        //   5명(또는 N명)의 AssigneeIssuePropertyVOs 의 issuePropertyVOs 를 활용해서 Set으로 만든 후, 유일한 List 뽑아냄
        HorizontalBarChartYAxisVO horizontalBarChartYAxisVO = toHorizontalBarChartYAxisVO(assigneeIssuePropertiesVOList);

        // 3. Chart의 Series 영역에 들어갈 VO 설정.
        List<HorizontalBarChartSeriesVO> horizontalBarChartSeries =
            toHorizontalBarChartSeries(assigneeIssuePropertiesVOList, horizontalBarChartYAxisVO);

        return HorizontalBarChartYAxisAndSeriesVO.builder()
            .yAxisVO(horizontalBarChartYAxisVO)
            .series(horizontalBarChartSeries)
            .build();
    }

    private List<HorizontalBarChartSeriesVO> toHorizontalBarChartSeries(
        List<AssigneeIssuePropertiesVO> assigneeIssuePropertyVOsList, HorizontalBarChartYAxisVO horizontalBarChartYAxisVO) {

        List<HorizontalBarChartSeriesVO> horizontalBarChartSeries = new ArrayList<>();

        List<String> propertyNames = horizontalBarChartYAxisVO.getData();

        for (AssigneeIssuePropertiesVO assigneeIssuePropertiesVO : assigneeIssuePropertyVOsList) {
            HorizontalBarChartSeriesVO.HorizontalBarChartSeriesVOBuilder horizontalBarChartSeriesVOBuilder =
                HorizontalBarChartSeriesVO.builder()
                    .name(assigneeIssuePropertiesVO.getName())
                    .type("bar");

            List<IssuePropertyVO> issueProperties = assigneeIssuePropertiesVO.getIssueProperties();
            Map<String, Long> issuePropertyMap = issueProperties.stream()
                .collect(Collectors.toMap(IssuePropertyVO::getName, IssuePropertyVO::getValue));

            horizontalBarChartSeriesVOBuilder.data(
                propertyNames.stream()
                    .map(propertyName -> issuePropertyMap.getOrDefault(propertyName, 0L))
                    .collect(Collectors.toList()));

            horizontalBarChartSeries.add(horizontalBarChartSeriesVOBuilder.build());
        }

        return horizontalBarChartSeries;
    }

    private AssigneeIssuePropertiesVO toAssigneeIssueProperties(DocumentAggregations documentAggregations, AssigneeDTO assigneeDTO) {
        AssigneeIssuePropertiesVO.AssigneeIssuePropertiesVOBuilder assigneeIssuePropertiesBuilder =
            AssigneeIssuePropertiesVO.builder()
                .accountId(assigneeDTO.getAccountId())
                .serverId(assigneeDTO.getServerId())
                .name(assigneeDTO.getName());

        if (assigneeDTO.getEmailAddress() != null && !assigneeDTO.getEmailAddress().isEmpty()) {
            String emailId = getEmailId(assigneeDTO.getEmailAddress());
            String nameWithEmailId = assigneeDTO.getName() + " ("+emailId+")";
            assigneeIssuePropertiesBuilder
                .name(nameWithEmailId)
                .emailAddress(assigneeDTO.getEmailAddress());
        }

        List<IssuePropertyVO> issueProperties = new ArrayList<>();

        List<DocumentBucket> deepestSubList = documentAggregations.deepestSubList();

        for (DocumentBucket documentBucket : deepestSubList) {
            IssuePropertyVO build = IssuePropertyVO.builder()
                .name(documentBucket.getDocFieldName())
                .value(documentBucket.getDocCount())
                .build();
            issueProperties.add(build);
        }

        return assigneeIssuePropertiesBuilder.issueProperties(issueProperties).build();

    }

    private HorizontalBarChartYAxisVO toHorizontalBarChartYAxisVO(List<AssigneeIssuePropertiesVO> assigneeIssuePropertyVOS) {
        Set<String> propertyNameSet = assigneeIssuePropertyVOS.stream()
            .flatMap(issueProperties -> issueProperties.getIssueProperties().stream())
            .map(IssuePropertyVO::getName)
            .collect(Collectors.toSet());

        List<String> propertyNames = new ArrayList<>(propertyNameSet);

        return HorizontalBarChartYAxisVO.builder()
            .type("category")
            .data(propertyNames)
            .build();
    }

    @Override
    public List<UniqueAssigneeVO> findAssigneesInfo(ResourceDTO resourceDTO) {

        ResourceAggrDTO resourceAggrDTO = new ResourceAggrDTO();
        resourceAggrDTO.setPdServiceAndIsReq(resourceDTO.getPdServiceAndIsReq());

        EsQuery esQuery = esQueryWithAssignee(resourceAggrDTO, null);

        Map<String, UniqueAssigneeVO> uniquedAssigneeMap = uniqueAssigneeMap(esQuery, resourceAggrDTO);

        return new ArrayList<>(uniquedAssigneeMap.values());
    }

    @Override
    public List<TreeMapWorkerVO> 작업자_별_요구사항_별_관여도_treemap(ResourceWithVersionIdNamesDTO resourceWithVersionIdNamesDTO) {
        Map<String, TreeMapWorkerVO> contributionMap = new HashMap<>();

        List<VersionIdNameDTO> versionIdNames = resourceWithVersionIdNamesDTO.getVersionIdNames();

        List<AlmIssueEntity> requirementIssues = this.findIssue(resourceWithVersionIdNamesDTO.getPdServiceAndIsReq());

        // 요구사항의 키를 모두 추출
        List<String> allReqKeys = requirementIssues.stream()
                .map(AlmIssueEntity::getKey)
                .collect(Collectors.toList());

        // 모든 하위 태스크를 한 번에 로드
        List<AlmIssueEntity> allSubTasks = this.findSubtaskIssues(allReqKeys);

        // 하위 태스크를 부모 키로 그룹화
        Map<String, List<AlmIssueEntity>> subTasksByParent = allSubTasks.stream()
                .filter(subtask -> subtask.getAssignee() != null)
                .collect(Collectors.groupingBy(AlmIssueEntity::getParentReqKey));

        requirementIssues.stream()
                .forEach(reqIssue -> {
                    String key = reqIssue.getKey();
                    String summary = reqIssue.getSummary() == null ? "해당 요구사항은 지라서버에서 조회가 되지 않는 상태입니다." : reqIssue.getSummary();
                    List<Long> pdServiceVersions = reqIssue.getPdServiceVersions();
                    String versionNames = pdServiceVersions.stream()
                            .map(versionId ->
                                    versionIdNames.stream()
                                            .filter(p -> p.getC_id().equals(versionId.toString()))
                                            .findFirst()
                                            .map(VersionIdNameDTO::getC_title).orElse(null))
                            .filter(Objects::nonNull)
                            .collect(Collectors.joining(","));

                    Optional.ofNullable(subTasksByParent.get(key)).orElse(Collections.emptyList()).stream() // List<AlmIssueEntity>
                            .forEach(subtask -> {
                                String assigneeId = subtask.getAssignee().getAccountId();
                                String displayName = subtask.getAssignee().getDisplayName();

                                TreeMapWorkerVO worker = contributionMap.computeIfAbsent(assigneeId, id -> {
                                    Map<String, Integer> dataMap = new HashMap<>();
                                    dataMap.put("totalInvolvedCount", 0);
                                    return TreeMapWorkerVO.builder()
                                            .id(assigneeId)
                                            .name(displayName)
                                            .data(dataMap)
                                            .children(new ArrayList<>())
                                            .build();
                                });

                                TreeMapTaskListVO taskList = worker.getChildren().stream()
                                        .filter(task -> task.getId().equals(key))
                                        .findFirst()
                                        .orElseGet(() -> {
                                            Map<String, Integer> dataList = new HashMap<>();
                                            dataList.put("involvedCount", 0);
//                                    TaskList newTask = new TaskList(key, "[ " + versionNames + " ] - " + summary, dataList);
                                            TreeMapTaskListVO newTask = TreeMapTaskListVO.builder()
                                                    .id(key).name("[ " + versionNames + " ] - " + summary)
                                                    .data(dataList)
                                                    .build();
                                            worker.getChildren().add(newTask);
                                            return newTask;
                                        });

                                taskList.getData().put("involvedCount", taskList.getData().get("involvedCount") + 1);
                                worker.getData().put("totalInvolvedCount", worker.getData().get("totalInvolvedCount") + 1);
                            });
                });

        return contributionMap.values().stream()
                .sorted((w1, w2) -> w2.getData().get("totalInvolvedCount").compareTo(w1.getData().get("totalInvolvedCount")))
                .limit(resourceWithVersionIdNamesDTO.getSize() > 0 ? resourceWithVersionIdNamesDTO.getSize() : Long.MAX_VALUE)
                .collect(toList());
    }

    private List<AlmIssueEntity> findIssue(PdServiceAndIsReqDTO pdServiceAndIsReqDto) {
        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("isReq", pdServiceAndIsReqDto.getIsReq()),
                        new TermsQueryFilter("pdServiceId", pdServiceAndIsReqDto.getPdServiceId()),
                        new TermsQueryFilter("pdServiceVersions", pdServiceAndIsReqDto.getPdServiceVersions())
                );
        ResourceDTO resourceDTO = new ResourceDTO();
        resourceDTO.setEsQuery(esQuery);

        return this.findRecentHits(resourceDTO).toDocs();
    }

    private List<AlmIssueEntity> findSubtaskIssues(List<String> parentReqKeys) {

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("parentReqKey", parentReqKeys)
                );
        ResourceDTO resourceDTO = new ResourceDTO();
        resourceDTO.setEsQuery(esQuery);

        return this.findRecentHits(resourceDTO).toDocs();
    }
}
