package com.arms.api.analysis.time.controller;

import com.arms.api.analysis.time.dto.TimeAggrDTO;
import com.arms.api.analysis.time.dto.TimeDTO;
import com.arms.api.analysis.time.service.AnalysisTime;
import com.arms.api.analysis.time.vo.AlmIssueStatByDateVO;
import com.arms.api.analysis.time.vo.RidgeLineVO;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping("/engine/analysis-time")
@Slf4j
@AllArgsConstructor
public class AnalysisTimeController {

    private final AnalysisTime analysisTime;

    // 히트맵
    @PostMapping("/updated-issue/last-year")
    public ResponseEntity<List<AlmIssueEntity>> getUpdatedIssueForLastYear(@RequestBody TimeDTO timeDTO) {
        return ResponseEntity.ok(analysisTime.getUpdatedIssueForLastYear(timeDTO));
    }

    // 스캐터 차트
    @PostMapping("/updated-issue/by-date")
    public ResponseEntity<AlmIssueStatByDateVO> getUpdatedIssueByDate(@RequestBody TimeAggrDTO timeAggrDTO) {
        return ResponseEntity.ok(analysisTime.getUpdatedIssueByDate(timeAggrDTO));
    }

    // 멀티 콤비네이션 차트
    @PostMapping("/updated-issue/status/by-date")
    public ResponseEntity<AlmIssueStatByDateVO> getUpdatedIssueByDateAndStatus(@RequestBody TimeAggrDTO timeAggrDTO) {
        return ResponseEntity.ok(analysisTime.getUpdatedIssueByDateAndStatus(timeAggrDTO));
    }

    // 이슈 타임라인 차트
    @PostMapping("/updated-timeline")
    public ResponseEntity<List<AlmIssueEntity>> getUpdatedIssueByDateRange(@RequestBody TimeAggrDTO timeAggrDTO) {
        return ResponseEntity.ok(analysisTime.getUpdatedIssueByDateRange(timeAggrDTO));
    }

    // 능선 차트 - 버전별 일자(업데이트)별 이슈
    @PostMapping("/updated-frequency")
    public ResponseEntity<List<RidgeLineVO>> getUpdatedFrequency(@RequestBody TimeAggrDTO timeAggrDTO) {
        return ResponseEntity.ok(analysisTime.getUpdatedFrequency(timeAggrDTO));
    }

}
