package com.arms.api.dashboard.service;

import com.arms.api.dashboard.model.dto.ResourceWithVersionIdNamesDTO;
import com.arms.api.dashboard.model.dto.VersionIdNameDTO;
import com.arms.api.dashboard.model.vo.TreeMapTaskListVO;
import com.arms.api.dashboard.model.vo.TreeMapWorkerVO;
import com.arms.api.dashboard.model.dto.DashboardAggrDTO;
import com.arms.api.dashboard.model.dto.DashboardDTO;
import com.arms.api.dashboard.model.vo.Top5ReqAssigneeVO;
import com.arms.api.dashboard.model.vo.AssigneeIssueCountVO;
import com.arms.api.dashboard.model.vo.AssigneePerVersionVO;
import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.util.model.dto.PdServiceAndIsReqDTO;
import com.arms.api.util.model.dto.request.지라이슈_기본_검색_집계_하위_요청;
import com.arms.egovframework.javaservice.esframework.esquery.EsBoolQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.ExistsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.model.dto.집계_하위_필드;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentBucket;
import com.arms.egovframework.javaservice.esframework.model.vo.버킷_집계_결과_목록_합계;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static java.util.stream.Collectors.toList;

@Slf4j
@Service("DashboardService")
@AllArgsConstructor
public class DashboardImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements Dashboard {

    @Override
    public 버킷_집계_결과_목록_합계 제품_버전_요구사항_이슈_집계(Long pdServiceId, 지라이슈_기본_검색_집계_하위_요청 지라이슈_일반_집계_요청) {

        EsQuery esQuery
                = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", pdServiceId),
                        new TermsQueryFilter("pdServiceVersions", 지라이슈_일반_집계_요청.getPdServiceVersionLinks()),
                        new TermQueryMust("isReq", 지라이슈_일반_집계_요청.getIsReq())
                );

//        return 지라이슈_저장소.aggregateRecentDocs(중첩_집계_쿼리_생성기.포괄(지라이슈_일반_집계_요청, esQuery).생성());
        // 임시 주석처리
        return null;
    }

    @Override
    public List<Top5ReqAssigneeVO> top5ReqAssignee(DashboardAggrDTO dashboardAggrDTO) {

        dashboardAggrDTO.set메인_그룹_필드("assignee.assignee_emailAddress.keyword");
        dashboardAggrDTO.set메인_그룹_필드명_별칭("emailAddress");
        dashboardAggrDTO.set크기(5);

        dashboardAggrDTO.set집계_하위_필드들(
            List.of(집계_하위_필드.builder()
                .하위_필드명("cReqLink")
                .하위_필드명_별칭("cReqLink")
                .build()
            )
        );

        EsBoolQuery[] esBoolQueries = Stream.of(
            new TermQueryMust("pdServiceId", dashboardAggrDTO.getPdServiceLink()),
            new TermsQueryFilter("pdServiceVersions", dashboardAggrDTO.getPdServiceVersionLinks()),
            new TermQueryMust("isReq", dashboardAggrDTO.getIsReq())
        ).toArray(EsBoolQuery[]::new);

        EsQuery esQuery = new EsQueryBuilder().bool(esBoolQueries);

        dashboardAggrDTO.setEsQuery(esQuery);
        DocumentAggregations aggregations = this.aggregateRecentDocs(dashboardAggrDTO);

        List<DocumentBucket> dicList = aggregations.deepestSubList();

        List<Top5ReqAssigneeVO> top5ReqAssigneeVOList = new ArrayList<>();

        for (DocumentBucket docBuckets : dicList) {
            String emailAddress = docBuckets.getDocFieldName("emailAddress");
            Long count = docBuckets.getDocFieldNameCount("emailAddress");
            String cReqLink = docBuckets.getDocFieldName("cReqLink");
            Top5ReqAssigneeVO top5ReqAssigneeVO = Top5ReqAssigneeVO.builder()
                .emailAddress(emailAddress)
                .reqCount(count)
                .reqLink(cReqLink)
                .build();
            top5ReqAssigneeVOList.add(top5ReqAssigneeVO);
        }
        return top5ReqAssigneeVOList;
    }

    @Override
    public List<TreeMapWorkerVO> 작업자_별_요구사항_별_관여도_treemap(ResourceWithVersionIdNamesDTO resourceWithVersionIdNamesDTO) {
        Map<String, TreeMapWorkerVO> contributionMap = new HashMap<>();

        List<VersionIdNameDTO> versionIdNames = resourceWithVersionIdNamesDTO.getVersionIdNames();

        List<AlmIssueEntity> requirementIssues = this.findIssue(resourceWithVersionIdNamesDTO.getPdServiceAndIsReq());

        // 요구사항의 키를 모두 추출
        List<String> allReqKeys = requirementIssues.stream()
                .map(AlmIssueEntity::getKey)
                .collect(Collectors.toList());

        // 모든 하위 태스크를 한 번에 로드
        List<AlmIssueEntity> allSubTasks = this.findSubtaskIssues(allReqKeys);

        // 하위 태스크를 부모 키로 그룹화
        Map<String, List<AlmIssueEntity>> subTasksByParent = allSubTasks.stream()
                .filter(subtask -> subtask.getAssignee() != null)
                .collect(Collectors.groupingBy(AlmIssueEntity::getParentReqKey));

        requirementIssues.stream()
                .forEach(reqIssue -> {
                    String key = reqIssue.getKey();
                    String summary = reqIssue.getSummary() == null ? "해당 요구사항은 지라서버에서 조회가 되지 않는 상태입니다." : reqIssue.getSummary();
                    List<Long> pdServiceVersions = reqIssue.getPdServiceVersions();
                    String versionNames = pdServiceVersions.stream()
                            .map(versionId ->
                                    versionIdNames.stream()
                                            .filter(p -> p.getC_id().equals(versionId.toString()))
                                            .findFirst()
                                            .map(VersionIdNameDTO::getC_title).orElse(null))
                            .filter(Objects::nonNull)
                            .collect(Collectors.joining(","));

                    Optional.ofNullable(subTasksByParent.get(key)).orElse(Collections.emptyList()).stream() // List<AlmIssueEntity>
                            .forEach(subtask -> {
                                String assigneeId = subtask.getAssignee().getAccountId();
                                String displayName = subtask.getAssignee().getDisplayName();

                                TreeMapWorkerVO worker = contributionMap.computeIfAbsent(assigneeId, id -> {
                                    Map<String, Integer> dataMap = new HashMap<>();
                                    dataMap.put("totalInvolvedCount", 0);
                                    return TreeMapWorkerVO.builder()
                                            .id(assigneeId)
                                            .name(displayName)
                                            .data(dataMap)
                                            .children(new ArrayList<>())
                                            .build();
                                });

                                TreeMapTaskListVO taskList = worker.getChildren().stream()
                                        .filter(task -> task.getId().equals(key))
                                        .findFirst()
                                        .orElseGet(() -> {
                                            Map<String, Integer> dataList = new HashMap<>();
                                            dataList.put("involvedCount", 0);
//                                    TaskList newTask = new TaskList(key, "[ " + versionNames + " ] - " + summary, dataList);
                                            TreeMapTaskListVO newTask = TreeMapTaskListVO.builder()
                                                    .id(key).name("[ " + versionNames + " ] - " + summary)
                                                    .data(dataList)
                                                    .build();
                                            worker.getChildren().add(newTask);
                                            return newTask;
                                        });

                                taskList.getData().put("involvedCount", taskList.getData().get("involvedCount") + 1);
                                worker.getData().put("totalInvolvedCount", worker.getData().get("totalInvolvedCount") + 1);
                            });
                });

        return contributionMap.values().stream()
                .sorted((w1, w2) -> w2.getData().get("totalInvolvedCount").compareTo(w1.getData().get("totalInvolvedCount")))
                .limit(resourceWithVersionIdNamesDTO.getSize() > 0 ? resourceWithVersionIdNamesDTO.getSize() : Long.MAX_VALUE)
                .collect(toList());
    }

    @Override
    public List<AssigneePerVersionVO> findVersionAssignees(DashboardDTO dashboardDTO) {

        DashboardAggrDTO dashboardAggrDTO = new DashboardAggrDTO();
        dashboardAggrDTO.setPdServiceAndIsReq(dashboardDTO.getPdServiceAndIsReq());

        EsQuery notReqIssueQuery = esQueryWithAssignee(dashboardAggrDTO, Boolean.FALSE);

        dashboardAggrDTO.set메인_그룹_필드("pdServiceVersions");
        dashboardAggrDTO.set메인_그룹_필드명_별칭("pdServiceVersions");
        dashboardAggrDTO.set집계_하위_필드들(
                List.of(
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assigneeID")
                                .하위_필드명("assignee.assignee_accountId.keyword")
                                .크기(5) // 버전별 5명씩
                                .build(),
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assigneeName")
                                .하위_필드명("assignee.assignee_displayName.keyword")
                                .build(),
                        집계_하위_필드.builder()
                                .하위_필드명_별칭("assigneeEmail")
                                .하위_필드명("assignee.assignee_emailAddress.keyword")
                                .build()
                )
        );
        dashboardAggrDTO.setEsQuery(notReqIssueQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(dashboardAggrDTO);

        Map<Long, AssigneePerVersionVO> versionIdVOMap = new HashMap<>();

        for (DocumentBucket documentBucket : documentAggregations.deepestSubList()) {

            Long versionId = Long.valueOf(documentBucket.getDocFieldName("pdServiceVersions"));
            AssigneeIssueCountVO assigneeIssueCountVO = AssigneeIssueCountVO
                    .builder()
                    .name(documentBucket.getDocFieldName("assigneeName"))
                    .accountId(documentBucket.getDocFieldName("assigneeID"))
                    .emailAddress(Optional.ofNullable(documentBucket.getDocFieldName("assigneeEmail")).orElse(""))
                    .issueCount(documentBucket.getDocCount())
                    .build();

            if (versionIdVOMap.containsKey(versionId)) {
                AssigneePerVersionVO assigneePerVersionVO = versionIdVOMap.get(versionId);
                List<AssigneeIssueCountVO> assigneeIssueCounts = assigneePerVersionVO.getAssigneeIssueCounts();
                assigneeIssueCounts.add(assigneeIssueCountVO);
            } else {

                AssigneePerVersionVO assigneePerVersionVO = AssigneePerVersionVO.builder()
                        .versionId(versionId)
                        .assigneeIssueCounts(new ArrayList<>(List.of(assigneeIssueCountVO)))
                        .build();
                versionIdVOMap.put(versionId, assigneePerVersionVO);
            }
        }

        return new ArrayList<>(versionIdVOMap.values());
    }

    private List<AlmIssueEntity> findIssue(PdServiceAndIsReqDTO pdServiceAndIsReqDto) {
        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("isReq", pdServiceAndIsReqDto.getIsReq()),
                        new TermsQueryFilter("pdServiceId", pdServiceAndIsReqDto.getPdServiceId()),
                        new TermsQueryFilter("pdServiceVersions", pdServiceAndIsReqDto.getPdServiceVersions())
                );
        DashboardDTO dashboardDTO = new DashboardDTO();
        dashboardDTO.setEsQuery(esQuery);

        return this.findRecentHits(dashboardDTO).toDocs();
    }

    private List<AlmIssueEntity> findSubtaskIssues(List<String> parentReqKeys) {
        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("parentReqKey", parentReqKeys)
                );

        DashboardDTO dashboardDTO = new DashboardDTO();
        dashboardDTO.setEsQuery(esQuery);

        return this.findRecentHits(dashboardDTO).toDocs();
    }

    private EsQuery esQueryWithAssignee(DashboardAggrDTO dashboardAggrDTO, Boolean isReq) {
        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermQueryMust("pdServiceId", dashboardAggrDTO.getPdServiceLink()),
                        new TermsQueryFilter("pdServiceVersions", dashboardAggrDTO.getPdServiceVersionLinks()),
                        new TermQueryMust("isReq", isReq),
                        new ExistsQueryFilter("assignee")
                );
        return esQuery;
    }
}
