package com.arms.api.fluentd.service;

import com.arms.api.search_engine.dto.SearchAggrDTO;
import com.arms.api.search_engine.dto.SearchDTO;
import com.arms.api.search_engine.vo.SearchAggrResultVO;
import com.arms.api.search_engine.vo.SearchResultVO;
import com.arms.api.util.model.dto.request.검색어_날짜포함_검색_요청;
import com.arms.api.fluentd.entity.FluentdEntity;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.model.dto.기본_정렬_요청;
import com.arms.egovframework.javaservice.esframework.esquery.filter.QueryStringFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.RangeQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.EsSortQuery;

import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.model.vo.DocumentSearchResult;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;

import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;


@Slf4j
@Service("fluentdSearch")
@AllArgsConstructor
public class FluentdSearchImpl extends EsCommonRepositoryWrapper<FluentdEntity> implements FluentdSearch {

    @Override
    public SearchAggrResultVO aggregateByFluentdLogName(SearchAggrDTO searchAggrDTO) {
        String startDate = convertTimeFormatOfFluentdLog(searchAggrDTO.getStartDate());
        String endDate = convertTimeFormatOfFluentdLog(searchAggrDTO.getEndDate());

        searchAggrDTO.set페이지(0);
        searchAggrDTO.set크기(5);
        searchAggrDTO.set메인_그룹_필드("@log_name");
        searchAggrDTO.set메인_그룹_필드명_별칭("logName");
        searchAggrDTO.set결과_갯수_기준_오름차순(false);

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new QueryStringFilter(searchAggrDTO.getSearchString()),
                        RangeQueryFilter.of("@timestamp").betweenDate(startDate, endDate)
                );

        searchAggrDTO.setEsQuery(esQuery);

        DocumentAggregations documentAggregations = this.aggregateDocs(searchAggrDTO);

        return SearchAggrResultVO.builder()
                .totalHits(documentAggregations.getTotalHits())
                .docBuckets(documentAggregations.docBuckets())
                .build();
    }


    @Override
    public SearchResultVO<FluentdEntity> searchFluentdLog(SearchDTO searchDTO) {

        String startDate = convertTimeFormatOfFluentdLog(searchDTO.getStartDate());
        String endDate = convertTimeFormatOfFluentdLog(searchDTO.getEndDate());

        searchDTO.set페이지(searchDTO.get페이지());
        searchDTO.set크기(searchDTO.get크기());

        String filterFieldName = getFilterFieldName(searchDTO.getFilter().getName());

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new QueryStringFilter(searchDTO.getSearchString()),
                        new TermsQueryFilter(   filterFieldName,
                                searchDTO.getFilter().getValues()),
                        RangeQueryFilter.of("@timestamp").betweenDate(startDate, endDate)
                ).sort(new EsSortQuery(
                        List.of(
                                기본_정렬_요청.builder().필드("@timestamp").정렬_기준("desc").build()
                        )
                ));

        searchDTO.setEsQuery(esQuery);

        DocumentSearchResult<FluentdEntity> hits = this.findHits(searchDTO);

        return SearchResultVO.<FluentdEntity>builder()
                .totalResultDocs(hits.getTotalHits())
                .searchResultDocs(hits.toHitDocs())
                .build();
    }


    @Override
    public void keepAliveConnection(){
        log.info("엘라스틱서치 커넥션 상태 유지");
        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                new TermsQueryFilter("id", "313")
            );
        SearchDTO searchDTO = new SearchDTO();
        searchDTO.set크기(1);
        searchDTO.setEsQuery(esQuery);
        this.findHits(searchDTO);
    }

    private String getFilterFieldName(String filterName) {
        if (filterName != null && !filterName.isBlank() && filterName.equals("log")) {
            return "@log_name";
        }
        return null;
    }

    private String convertTimeFormatOfFluentdLog(String inputTime) {
        if (inputTime != null && !inputTime.isEmpty()) {
            LocalDateTime localDateTime  = LocalDateTime.parse(inputTime, DateTimeFormatter.ISO_DATE_TIME);

            ZonedDateTime zonedDateTime = localDateTime.atZone(ZoneOffset.ofHours(0));

            return zonedDateTime.format(DateTimeFormatter.ofPattern("uuuu-MM-dd'T'HH:mm:ss.SSSSSSSSSZZZZZ"));
        } else {
            return null;
        }

    }
}
