package com.arms.api.issue.almapi.controller;

import com.arms.api.issue.almapi.model.vo.지라이슈_데이터;
import com.arms.api.issue.almapi.model.dto.AlmIssueCreationDTO;
import com.arms.api.issue.almapi.service.이슈전략_호출;
import com.arms.api.util.redis.RedisUtil;
import com.arms.egovframework.javaservice.esframework.erros.response.ErrorCode;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/{connectId}/jira/issue")
public class IssueController {
    
    @Autowired
    private 이슈전략_호출 이슈전략_호출;

    @Autowired
    private RedisUtil redisUtil;

    @ResponseBody
    @RequestMapping(
            value = {"/list"},
            method = {RequestMethod.GET}
    )
    public List<지라이슈_데이터> getIssueList(@PathVariable("connectId") Long connectId,
                                            @RequestParam("projectKeyOrId") String projectKeyOrId) {

        if (StringUtils.isEmpty((projectKeyOrId))) {
            log.error("getIssueList Error projectKeyOrId {}", ErrorCode.파라미터_NULL_오류.getErrorMsg());
            throw new IllegalArgumentException("getIssueList Error projectKeyOrId " + ErrorCode.파라미터_NULL_오류.getErrorMsg());
        }

        return 이슈전략_호출.getIssueList(connectId, projectKeyOrId);
    }

    @ResponseBody
    @RequestMapping(
            value = {""},
            method = {RequestMethod.POST}
    )
    public 지라이슈_데이터 createIssue(@PathVariable("connectId") Long connectId,
                                    @RequestBody AlmIssueCreationDTO almIssueCreationDTO) {

        if (almIssueCreationDTO == null) {
            log.error("createIssue Error almIssueCreationDTO is null");
            throw new IllegalArgumentException("createIssue Error almIssueCreationDTO is null");
        }

        return 이슈전략_호출.createIssue(connectId, almIssueCreationDTO);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/{issueKeyOrId}"},
            method = {RequestMethod.PUT}
    )
    public Map<String,Object> 이슈_수정하기(@PathVariable("connectId") Long 연결_아이디,
                                            @PathVariable("issueKeyOrId") String 이슈_키_또는_아이디,
                                            @RequestBody AlmIssueCreationDTO almIssueCreationDTO) {

        log.info("이슈_수정하기 :: 연결_아이디 :: {} :: 지라이슈생성_데이터 :: {}",
                                            연결_아이디, almIssueCreationDTO.toString());
        return 이슈전략_호출.이슈_수정하기(연결_아이디, 이슈_키_또는_아이디, almIssueCreationDTO);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/{issueKeyOrId}/{statusId}"},
            method = {RequestMethod.PUT, RequestMethod.POST}
    )
    public Map<String,Object> 이슈_상태_변경하기(@PathVariable("connectId") Long 연결_아이디,
                                                @PathVariable("issueKeyOrId") String 이슈_키_또는_아이디,
                                                @PathVariable("statusId") String 상태_아이디) {

        log.info("이슈_상태_변경하기 :: 연결_아이디 :: {} :: 이슈_키_또는_아이디 :: {} :: 상태_아이디 :: {}",
                연결_아이디, 이슈_키_또는_아이디, 상태_아이디);
        return 이슈전략_호출.이슈_상태_변경하기(연결_아이디, 이슈_키_또는_아이디, 상태_아이디);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/{issueKeyOrId}"},
            method = {RequestMethod.DELETE}
    )
    public Map<String,Object> 이슈_삭제하기(@PathVariable("connectId") Long 연결_아이디,
                                            @PathVariable("issueKeyOrId") String 이슈_키_또는_아이디) {

        log.info("이슈_삭제하기 :: 연결_아이디 :: {} :: 이슈_키_또는_아이디 :: {}",
                                            연결_아이디, 이슈_키_또는_아이디);
        return 이슈전략_호출.이슈_삭제하기(연결_아이디, 이슈_키_또는_아이디);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/{issueKeyOrId}"},
            method = {RequestMethod.GET}
    )
    public 지라이슈_데이터 이슈_상세정보_가져오기(@PathVariable("connectId") Long 연결_아이디,
                                                @PathVariable("issueKeyOrId") String 이슈_키_또는_아이디) throws Exception {

        log.info("이슈_상세정보_가져오기 :: 연결_아이디 : {}, 이슈_키_또는_아이디 : {}",
                                                    연결_아이디, 이슈_키_또는_아이디);
        return 이슈전략_호출.이슈_상세정보_가져오기(연결_아이디, 이슈_키_또는_아이디);
    }

    // 연결 이슈 가져오기
    @ResponseBody
    @RequestMapping(
            value = {"/link/{issueKeyOrId}"},
            method = {RequestMethod.GET}
    )
    public List<지라이슈_데이터> 연결이슈_가져오기(@PathVariable("connectId") Long 연결_아이디,
                                    @PathVariable("issueKeyOrId") String 이슈_키_또는_아이디) throws Exception {

        log.info("연결이슈_가져오기 :: 연결_아이디 : {}, 이슈_키_또는_아이디 : {}",
                                                    연결_아이디, 이슈_키_또는_아이디);
        return 이슈전략_호출.연결이슈_가져오기(연결_아이디, 이슈_키_또는_아이디);
    }

    @ResponseBody
    @RequestMapping(
            value = {"/subtask/{issueKeyOrId}"},
            method = {RequestMethod.GET}
    )
    public List<지라이슈_데이터> 서브테스크_가져오기(@PathVariable("connectId") Long 연결_아이디,
                                                    @PathVariable("issueKeyOrId") String 이슈_키_또는_아이디) throws Exception {

        log.info("서브테스크_가져오기 :: 연결_아이디 : {}, 이슈_키_또는_아이디 : {}",
                                                연결_아이디, 이슈_키_또는_아이디);
        return 이슈전략_호출.서브테스크_가져오기(연결_아이디, 이슈_키_또는_아이디);
    }


    @ResponseBody
    @RequestMapping(
            value = {"/increment/subtask/{issueKeyOrId}"},
            method = {RequestMethod.GET}
    )
    public List<지라이슈_데이터> 증분서브테스크_가져오기(@PathVariable("connectId") Long 연결_아이디,
                                                        @PathVariable("issueKeyOrId") String 이슈_키_또는_아이디,
                                        @RequestParam(value = "startDate",required = false) String startDate,
                                       @RequestParam(value = "endDate",required = false) String endDate) {

        log.info("증분서브테스크_가져오기 :: 연결_아이디 : {}, 이슈_키_또는_아이디 : {}",
                                                연결_아이디, 이슈_키_또는_아이디);
        return 이슈전략_호출.증분서브테스크_가져오기(연결_아이디, 이슈_키_또는_아이디, startDate, endDate);
    }

    @RequestMapping(
            value = {"/getStatusMappingData"},
            method={RequestMethod.GET}
    )
    public ResponseEntity<?> getStatusMappingData(@RequestParam String key) {
        return ResponseEntity.ok(redisUtil.getHashData(key));
    }
}
