package com.arms.api.report.service;

import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.report.model.AssigneeIssueCountVO;
import com.arms.api.report.model.FullDataRequestDTO;
import com.arms.api.report.model.FullDataResponseDTO;
import com.arms.api.report.model.작업자_정보;
import com.arms.api.serverinfo.service.서버정보_서비스;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsBoolQuery;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.ExistsQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.RangeQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.model.dto.기본_검색_요청;
import com.arms.egovframework.javaservice.esframework.model.dto.기본_검색_집계_하위_요청;
import com.arms.egovframework.javaservice.esframework.model.dto.집계_하위_필드;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentBucket;
import com.arms.egovframework.javaservice.esframework.esquery.must.TermQueryMust;
import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service("리포트_서비스")
@AllArgsConstructor
public class ReportServiceImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements ReportService {

    private 서버정보_서비스 서버정보_서비스;

    @Override
    public List<작업자_정보> 작업자_목록_가져오기() {

        Map<String, String> 서버_연결아이디_유형_맵 = 서버정보_서비스.서버_연결아이디_유형_맵();

        기본_검색_집계_하위_요청 집계_요청 = new 기본_검색_집계_하위_요청();
        집계_요청.set메인_그룹_필드("assignee.assignee_emailAddress.keyword");
        집계_요청.set메인_그룹_필드명_별칭("emailAddress");
        집계_요청.set집계_하위_필드들(
            List.of(
                집계_하위_필드.builder()
                    .하위_필드명_별칭("jiraServerId")
                    .하위_필드명("jira_server_id")
                    .build()
                ,
                집계_하위_필드.builder()
                    .하위_필드명_별칭("displayName")
                    .하위_필드명("assignee.assignee_displayName.keyword")
                    .build()
                ,
                집계_하위_필드.builder()
                    .하위_필드명_별칭("accountId")
                    .하위_필드명("assignee.assignee_accountId.keyword")
                    .build()
            )
        );

        EsQuery esQuery = new EsQueryBuilder()
            .bool( new ExistsQueryFilter("assignee") );

        집계_요청.setEsQuery(esQuery);
        DocumentAggregations documentAggregations = this.aggregateRecentDocs(집계_요청);

        List<DocumentBucket> deepestSubList = documentAggregations.deepestSubList();

        Set<작업자_정보> 작업자_목록 = new HashSet<>();

        if (documentAggregations.getTotalHits() == 0) {
            log.info("[ 리포트_서비스_프로세스 :: 작업자_목록_가져오기 ] :: 작업자 정보가 없습니다. 사이즈 => 0");
        }else{

            for (DocumentBucket documentBucket : deepestSubList) {

                작업자_정보 작업자 = new 작업자_정보();

                작업자.setEmailAddress(documentBucket.getDocFieldName("emailAddress"));

                작업자.setServerType(서버_연결아이디_유형_맵.get(documentBucket.getDocFieldName("jiraServerId")));

                작업자.setDisplayName(documentBucket.getDocFieldName("displayName"));

                작업자.setAccountId(documentBucket.getDocFieldName("accountId"));

                작업자_목록.add(작업자);
            }

        }

        if(작업자_목록.isEmpty()) {
            return new ArrayList<>(Collections.emptyList());
        } else {
            return new ArrayList<>(작업자_목록);
        }
    }

    private static List<EsBoolQuery> 빌더에_넣을_공통_쿼리_목록_세팅(FullDataRequestDTO fullDataRequestDTO) {

        List<EsBoolQuery> queries = new ArrayList<>();
        if (fullDataRequestDTO.getPdServiceId() != null) {
            queries.add(new TermsQueryFilter("pdServiceId", fullDataRequestDTO.getPdServiceId()));
        }

        if (fullDataRequestDTO.getPdServiceVersionIds() != null && !fullDataRequestDTO.getPdServiceVersionIds().isEmpty()) {
            queries.add(new TermsQueryFilter("pdServiceVersions", fullDataRequestDTO.getPdServiceVersionIds()));
        }

        if (fullDataRequestDTO.getAlmProjectUrls() != null && !fullDataRequestDTO.getAlmProjectUrls().isEmpty()) {
            queries.add(new TermsQueryFilter("project.project_self.keyword", fullDataRequestDTO.getAlmProjectUrls()));
        }

        // 작업자
        if (fullDataRequestDTO.getEmailAddress() != null && !fullDataRequestDTO.getEmailAddress().isEmpty()) {
            queries.add(new TermsQueryFilter("assignee.assignee_emailAddress.keyword", fullDataRequestDTO.getEmailAddress()));
        }

        if (fullDataRequestDTO.getStartDate() != null && fullDataRequestDTO.getEndDate() == null) {
            queries.add(new RangeQueryFilter("updated",
                fullDataRequestDTO.getStartDate(),null,"gte"));
        }
        else if (fullDataRequestDTO.getStartDate() == null && fullDataRequestDTO.getEndDate() != null) {
            queries.add(new RangeQueryFilter("updated",
                    null, fullDataRequestDTO.getEndDate(),"lte"));
        }
        else if (fullDataRequestDTO.getStartDate() != null && fullDataRequestDTO.getEndDate() != null) {
            queries.add(new RangeQueryFilter("updated",
                            fullDataRequestDTO.getStartDate(),fullDataRequestDTO.getEndDate(),"fromto"));
        }
        return queries;
    }

    private EsQueryBuilder 쿼리_빌더_생성(List<EsBoolQuery> 쿼리_목록) {
        EsQueryBuilder esQueryBuilder = new EsQueryBuilder();
        // bool 메서드 호출 시 리스트를 배열로 변환하여 전달
        if (!쿼리_목록.isEmpty()) {
            esQueryBuilder.bool(쿼리_목록.toArray(new EsBoolQuery[0]));

        }

        return esQueryBuilder;
    }

    @Override
    public FullDataResponseDTO 이슈_목록_가져오기_스크롤API(FullDataRequestDTO fullDataRequestDTO) {

        기본_검색_요청 기본_검색_요청 = new 기본_검색_요청();
        기본_검색_요청.set페이지(fullDataRequestDTO.getPage());
        기본_검색_요청.set크기(fullDataRequestDTO.getSize());

        EsQuery esQuery = new EsQueryBuilder()
            .bool(
                  new TermsQueryFilter("pdServiceId", fullDataRequestDTO.getPdServiceId())
                , new TermsQueryFilter("pdServiceVersions", fullDataRequestDTO.getPdServiceVersionIds())
                , new TermsQueryFilter("project.project_self.keyword", fullDataRequestDTO.getAlmProjectUrls())
                , new TermsQueryFilter("assignee.assignee_emailAddress.keyword", fullDataRequestDTO.getEmailAddress())
                , RangeQueryFilter.of("updated").betweenDate(fullDataRequestDTO.getStartDate(),fullDataRequestDTO.getEndDate())
            );

        기본_검색_요청.setEsQuery(esQuery);
        List<AlmIssueEntity> 스크롤API_활용_이슈_목록 
                = this.findRecentHits(기본_검색_요청).toDocs(); // 스크롤API 필요
//        List<AlmIssueEntity> 스크롤API_활용_이슈_목록
//                = 지라이슈_저장소.findRecentDocsByScrollApi(기본_쿼리_생성기.기본검색(기본_검색_요청, esQuery).생성());

        FullDataResponseDTO 결과 = new FullDataResponseDTO();
        결과.setTotalHits((long) 스크롤API_활용_이슈_목록.size());
        결과.setIssueEntityList(스크롤API_활용_이슈_목록);
        return 결과;
    }

    // 실사용
    @Override
    public FullDataResponseDTO 이슈_목록_가져오기(FullDataRequestDTO fullDataRequestDTO) {

        List<AlmIssueEntity> 모든_이슈_목록 = new ArrayList<>();
        // upperKey 필드가 없는 지라이슈_엔티티를 담을 리스트
        List<AlmIssueEntity> ALM_생성_연결이슈 = new ArrayList<>();
        // upperKey 필드가 있는 지라이슈_엔티티를 담을 리스트
        List<AlmIssueEntity> 하위이슈_목록 = new ArrayList<>();
        List<AlmIssueEntity> 하위이슈만_존재하는_이슈의_요구사항이슈_목록 = new ArrayList<>();
        Set<String> 하위이슈_연결이슈_docIdSet = new HashSet<>();
        // 요구사항 이슈의 연결이슈 docIdSet
        Set<String> linkedIssuesDocIdSet = new HashSet<>();

        기본_검색_요청 기본_검색_요청 = new 기본_검색_요청();
        기본_검색_요청.set페이지(fullDataRequestDTO.getPage());
        기본_검색_요청.set크기(fullDataRequestDTO.getSize());

        // 당사자가 assignee 로 들어가 있는 경우
        List<EsBoolQuery> 공통_쿼리_목록 = 빌더에_넣을_공통_쿼리_목록_세팅(fullDataRequestDTO);

        // 요구사항 목록 조회
        List<EsBoolQuery> 요구사항_쿼리_목록 = new ArrayList<>(공통_쿼리_목록);
        요구사항_쿼리_목록.add(new TermQueryMust("isReq", true));
        EsQueryBuilder 요구사항_이슈_빌더 = 쿼리_빌더_생성(요구사항_쿼리_목록);
        기본_검색_요청.setEsQuery(요구사항_이슈_빌더);
        List<AlmIssueEntity> 요구사항_목록 = this.findRecentDocsByScrollApi(기본_검색_요청); // 스크롤API 필요

        // 하위이슈 목록 조회, ALM 직접 생성 연결이슈
        List<EsBoolQuery> 하위이슈_ALM직접연결이슈_쿼리_목록 = new ArrayList<>(공통_쿼리_목록);
        하위이슈_ALM직접연결이슈_쿼리_목록.add(new TermQueryMust("isReq",false));
        EsQueryBuilder 하위이슈_빌더 = 쿼리_빌더_생성(하위이슈_ALM직접연결이슈_쿼리_목록);
        기본_검색_요청.setEsQuery(하위이슈_빌더);
        List<AlmIssueEntity> 하위이슈_ALM직접연결이슈_목록 = this.findRecentDocsByScrollApi(기본_검색_요청); // 스크롤API 필요


        // for 반복문을 사용해 upperKey 여부에 따라, 하위이슈 연결이슈 분리
        for (AlmIssueEntity entity : 하위이슈_ALM직접연결이슈_목록) {
            String upperKey = entity.getUpperKey();
            하위이슈_연결이슈_docIdSet.add(entity.getId());
            if (upperKey == null || upperKey.isEmpty()) {
                ALM_생성_연결이슈.add(entity);  // upperKey가 없으면
            } else {
                하위이슈_목록.add(entity);  // upperKey가 있으면
            }
        }

        // 요구사항 Key Set
        Set<String> parentReqKeySet = 요구사항_목록.stream()
                .map(AlmIssueEntity::getKey).collect(Collectors.toSet());

        // 하위이슈만 있어서, 부모 요구사항이 필요한 경우
        List<String> 중복제거한_하위이슈의_부모키 = new ArrayList<>(
                하위이슈_목록.stream()
                        .map(AlmIssueEntity::getParentReqKey)
                        .collect(Collectors.toSet()));

        List<String> 하위이슈만_존재하는경우의_부모키 = 중복제거한_하위이슈의_부모키.stream()
                .filter(key -> !parentReqKeySet.contains(key))
                .collect(Collectors.toList());

        if (!하위이슈만_존재하는경우의_부모키.isEmpty()) {

            EsQuery esQuery_for_parentIssues = new EsQueryBuilder()
                .bool(
                    new TermQueryMust("isReq", true),
                    new TermsQueryFilter("key", 하위이슈만_존재하는경우의_부모키)
                );

            기본_검색_요청.setEsQuery(esQuery_for_parentIssues);
            하위이슈만_존재하는_이슈의_요구사항이슈_목록 = this.findRecentDocsByScrollApi(기본_검색_요청); // 스크롤API 필요

        } else {
            log.info("하위이슈만 존재하는 경우 없음.");
        }

        for (AlmIssueEntity entity : 요구사항_목록) {
            List<String> linkedIssues = entity.getLinkedIssues();
            if (linkedIssues != null && linkedIssues.size() > 0) {
                // linkedIssues 배열을 Set에 추가하여 중복 제거
                for (String docId : linkedIssues) {
                    linkedIssuesDocIdSet.add(docId);
                }
            }
        }

        // 차집합 ( 요구사항의 연결이슈 배열 - 하위이슈_연결이슈_docIdSet) :: 이미 조회한 doc 제외
        linkedIssuesDocIdSet.removeAll(하위이슈_연결이슈_docIdSet);
        // 연결이슈만 존재하고, 연결이슈를 필요로 하는 요구사항이슈가 없는 경우
        List<String> linkedIssuesDocIdList = new ArrayList<>(linkedIssuesDocIdSet);

        EsQuery esQuery_for_linkedIssues = new EsQueryBuilder()
                .bool(  new TermsQueryFilter("id", linkedIssuesDocIdList),
                        new TermQueryMust("isReq",false)
                );
        기본_검색_요청.setEsQuery(esQuery_for_linkedIssues);
        List<AlmIssueEntity> 요구사항_연결이슈_목록 = this.findRecentDocsByScrollApi(기본_검색_요청); // 스크롤API 필요

        모든_이슈_목록.addAll(요구사항_목록);
        if (!하위이슈만_존재하는_이슈의_요구사항이슈_목록.isEmpty()) {
            모든_이슈_목록.addAll(하위이슈만_존재하는_이슈의_요구사항이슈_목록);
        }
        모든_이슈_목록.addAll(하위이슈_목록);
        모든_이슈_목록.addAll(ALM_생성_연결이슈);
        if (!요구사항_연결이슈_목록.isEmpty()) {
            모든_이슈_목록.addAll(요구사항_연결이슈_목록);
        }
        log.info("[리포트_서비스_프로세스 :: 이슈_목록_가져오기2 ] :: 요구사항 이슈 총 갯수 => {}", 요구사항_목록.size() + 하위이슈만_존재하는_이슈의_요구사항이슈_목록.size());

        FullDataResponseDTO 결과 = new FullDataResponseDTO();
        결과.setTotalHits((long) 모든_이슈_목록.size());
        결과.setIssueEntityList(모든_이슈_목록);
        log.info("[리포트_서비스_프로세스 :: 이슈_목록_가져오기2 ] :: 이슈 총 갯수 => {}", 모든_이슈_목록.size());
        return 결과;
    }

    @Override
    public FullDataResponseDTO getReqIssueList(FullDataRequestDTO fullDataRequestDTO){
        기본_검색_요청 기본_검색_요청 = new 기본_검색_요청();
        기본_검색_요청.set페이지(fullDataRequestDTO.getPage());
        기본_검색_요청.set크기(fullDataRequestDTO.getSize());

        List<EsBoolQuery> commonQueryList = 빌더에_넣을_공통_쿼리_목록_세팅(fullDataRequestDTO);

        commonQueryList.add(new TermsQueryFilter("pdServiceId", fullDataRequestDTO.getPdServiceIds()));

        List<EsBoolQuery> reqQueryList = new ArrayList<>(commonQueryList);
        reqQueryList.add(new TermQueryMust("isReq", true));
        EsQueryBuilder reqIssueBuilder = 쿼리_빌더_생성(reqQueryList);
        기본_검색_요청.setEsQuery(reqIssueBuilder);
        List<AlmIssueEntity> reqList = this.findRecentDocsByScrollApi(기본_검색_요청);

        FullDataResponseDTO result = new FullDataResponseDTO();
        result.setTotalHits((long) reqList.size());
        result.setIssueEntityList(reqList);
        return result;
    }

    @Override
    public AssigneeIssueCountVO getAssigneeAggregationList(FullDataRequestDTO fullDataRequestDTO) {

        기본_검색_집계_하위_요청 requestAggregation = new 기본_검색_집계_하위_요청();
        requestAggregation.set메인_그룹_필드("assignee.assignee_emailAddress.keyword");
        requestAggregation.set메인_그룹_필드명_별칭("assigneeEmail");
        requestAggregation.set페이지(fullDataRequestDTO.getPage());
        requestAggregation.set크기(fullDataRequestDTO.getSize());

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new TermsQueryFilter("pdServiceId", fullDataRequestDTO.getPdServiceIds()),
                        new TermsQueryFilter("pdServiceVersions", fullDataRequestDTO.getPdServiceVersionIds()),
                        new ExistsQueryFilter("assignee")
                );

        requestAggregation.setEsQuery(esQuery);

        DocumentAggregations documentAggregations = this.aggregateRecentDocs(requestAggregation);

        List<DocumentBucket> documentBuckets = documentAggregations.deepestSubList();

        long totalIssueCount = documentAggregations.getTotalHits();
        long totalAssigneeCount = documentBuckets.size();

        Map<String, Long> assigneeCounts = new HashMap<>();
        for (DocumentBucket documentBucket : documentBuckets) {
            assigneeCounts.put(documentBucket.getDocFieldName("assigneeEmail"), documentBucket.getDocCount());
        }

        double avgIssuesPerAssignee = totalAssigneeCount > 0
                ? (double) totalIssueCount / totalAssigneeCount
                : 0.0;

        long maxIssues = assigneeCounts.values().stream()
                .mapToLong(Long::longValue)
                .max()
                .orElse(0L);

        long minIssues = assigneeCounts.values().stream()
                .mapToLong(Long::longValue)
                .min()
                .orElse(0L);

        return new AssigneeIssueCountVO(
                totalIssueCount,
                totalAssigneeCount,
                avgIssuesPerAssignee,
                maxIssues,
                minIssues,
                assigneeCounts
        );
    }


}
