package com.arms.api.search_engine.service;

import com.arms.api.issue.almapi.model.entity.AlmIssueEntity;
import com.arms.api.search_engine.dto.SearchAggrDTO;
import com.arms.api.search_engine.dto.SearchDTO;
import com.arms.api.search_engine.vo.SearchAggrResultVO;
import com.arms.api.search_engine.vo.SearchResultVO;
import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.QueryStringFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.RangeQueryFilter;
import com.arms.egovframework.javaservice.esframework.model.vo.DocumentSearchResult;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;

import com.arms.egovframework.javaservice.esframework.repository.EsCommonRepositoryWrapper;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import static com.arms.egovframework.javaservice.esframework.factory.creator.중첩_집계_쿼리_생성기.포괄V2;

@Slf4j
@Service
@AllArgsConstructor
public class AlmIssueSearchImpl extends EsCommonRepositoryWrapper<AlmIssueEntity> implements AlmIssueSearch {

    @Override
    public SearchResultVO<AlmIssueEntity> searchAlmIssue(SearchDTO searchDTO) {

        String startDate = searchDTO.getStartDate();
        String endDate = searchDTO.getEndDate();

        searchDTO.set페이지(searchDTO.get페이지());
        searchDTO.set크기(searchDTO.get크기());

        String filterFieldName = getFilterFieldName(searchDTO.getFilter().getName());

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new QueryStringFilter(searchDTO.getSearchString()),
                        new TermsQueryFilter(   filterFieldName,
                                                searchDTO.getFilter().getValues()),
                        RangeQueryFilter.of("@timestamp").betweenDate(startDate, endDate)
                );

        searchDTO.setEsQuery(esQuery);

        DocumentSearchResult<AlmIssueEntity> hits = this.findHits(searchDTO);

        return SearchResultVO.<AlmIssueEntity>builder()
                .totalResultDocs(hits.getTotalHits())
                .searchResultDocs(hits.toHitDocs())
                .build();
    }

    private String getFilterFieldName(String filterName) {
        if (filterName != null && !filterName.isBlank() && filterName.equals("project")) {
            return "project.project_name.keyword";
        }
        return null;
    }

    // 포괄V2용
    @Override
    public SearchAggrResultVO aggregateByAlmProjectName(SearchAggrDTO searchAggrDTO) {

        String startDate = searchAggrDTO.getStartDate();
        String endDate = searchAggrDTO.getEndDate();

        searchAggrDTO.set페이지(0);
        searchAggrDTO.set크기(5);
        searchAggrDTO.set메인_그룹_필드("project.project_name.keyword");
        searchAggrDTO.set메인_그룹_필드명_별칭("projectName");
        searchAggrDTO.set결과_갯수_기준_오름차순(false);

        EsQuery esQuery = new EsQueryBuilder()
                .bool(
                        new QueryStringFilter(searchAggrDTO.getSearchString()),
                        RangeQueryFilter.of("@timestamp").betweenDate(startDate, endDate)
                );

        DocumentAggregations documentAggregations = this.aggregateDocs(searchAggrDTO);

        return SearchAggrResultVO.builder()
                .totalHits(documentAggregations.getTotalHits())
                .docBuckets(documentAggregations.docBuckets())
                .build();
    }

}
