package com.arms.egovframework.javaservice.esframework.repository;

import com.arms.egovframework.javaservice.esframework.esquery.EsQuery;
import com.arms.egovframework.javaservice.esframework.annotation.ElasticSearchIndex;
import com.arms.egovframework.javaservice.esframework.annotation.Recent;
import com.arms.egovframework.javaservice.esframework.annotation.RollingIndexName;
import com.arms.egovframework.javaservice.esframework.esquery.EsQueryBuilder;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermQueryFilter;
import com.arms.egovframework.javaservice.esframework.esquery.filter.TermsQueryFilter;
import com.arms.egovframework.javaservice.esframework.factory.creator.기본_쿼리_생성기;
import com.arms.egovframework.javaservice.esframework.model.vo.v2.DocumentAggregations;
import com.arms.egovframework.javaservice.esframework.model.vo.메트릭_집계_결과_목록_합계;
import com.arms.egovframework.javaservice.esframework.model.vo.DocumentSearchResult;
import com.arms.egovframework.javaservice.esframework.model.vo.버킷_집계_결과_목록_합계;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.TermsQueryBuilder;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.data.annotation.Id;
import org.springframework.data.elasticsearch.NoSuchIndexException;
import org.springframework.data.elasticsearch.annotations.Document;
import org.springframework.data.elasticsearch.core.*;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.NativeSearchQuery;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.data.elasticsearch.repository.support.ElasticsearchEntityInformation;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import static com.arms.egovframework.javaservice.esframework.util.ReflectionUtil.*;
import static java.util.stream.Collectors.*;

@Slf4j
public class 공통저장소_구현체<T,ID extends Serializable> extends SimpleElasticsearchWrapperRepository<T,ID> {

    private final Class<T> 엔티티클래스;

    private final ElasticsearchOperations 엘라스틱서치오퍼레이션스;

    public 공통저장소_구현체(ElasticsearchEntityInformation<T, ID> metadata
            , ElasticsearchOperations 엘라스틱서치오퍼레이션스) {
        super(metadata, 엘라스틱서치오퍼레이션스);
        this.엔티티클래스 = metadata.getJavaType();
        this.엘라스틱서치오퍼레이션스 = 엘라스틱서치오퍼레이션스;
    }

    @Override
    public 메트릭_집계_결과_목록_합계 metricAggregateDocs(Query query) {
        return new 메트릭_집계_결과_목록_합계(엘라스틱서치오퍼레이션스.search(query, 엔티티클래스));
    }

    @Override
    public DocumentAggregations aggregateRecentDocs(Query query,V2 v2) {
        NativeSearchQuery nativeSearchQuery = recentTrueQueryBuilder((NativeSearchQuery)query);
        return new DocumentAggregations(엘라스틱서치오퍼레이션스.search(nativeSearchQuery, 엔티티클래스));
    }

    @Override
    public DocumentAggregations aggregateDocs(Query query,V2 v2) {
        return new DocumentAggregations(엘라스틱서치오퍼레이션스.search(query, 엔티티클래스));
    }

    @Override
    public DocumentSearchResult<T> findAllHits() {
        NativeSearchQueryBuilder nativeSearchQueryBuilder = new NativeSearchQueryBuilder();
        return new DocumentSearchResult<>(엘라스틱서치오퍼레이션스.search(nativeSearchQueryBuilder.build(), 엔티티클래스),엔티티클래스);
    }

    @Override
    public DocumentSearchResult<T> findHits(Query query) {
        return new DocumentSearchResult<>(엘라스틱서치오퍼레이션스.search(query, 엔티티클래스),엔티티클래스);
    }

    @Override
    public DocumentSearchResult<T> findHits(Query query, String newIndex) {
        return new DocumentSearchResult<>(엘라스틱서치오퍼레이션스.search(query, 엔티티클래스, IndexCoordinates.of(newIndex)),엔티티클래스);
    }

    @Override
    public DocumentSearchResult<T> findRecentAllHits() {

        String recentFieldName = fieldInfo(엔티티클래스, Recent.class).getName();
        TermsQueryBuilder termsQueryBuilder = QueryBuilders.termsQuery(recentFieldName, true);

        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
        boolQueryBuilder.must(termsQueryBuilder);

        NativeSearchQueryBuilder nativeSearchQueryBuilder = new NativeSearchQueryBuilder();
        nativeSearchQueryBuilder.withQuery(boolQueryBuilder);

        return new DocumentSearchResult<>(엘라스틱서치오퍼레이션스.search(nativeSearchQueryBuilder.build(), 엔티티클래스),엔티티클래스);
    }


    @Override
    public DocumentSearchResult<T> findRecentHits(Query query) {

        try{
            ElasticSearchIndex annotation = AnnotationUtils.findAnnotation(엔티티클래스, ElasticSearchIndex.class);

            if(annotation==null){
                log.warn("ElasticSearchIndex annotatoin is null");
            }

            return new DocumentSearchResult<>(
                엘라스틱서치오퍼레이션스.search(recentTrueQueryBuilder((NativeSearchQuery)query), 엔티티클래스),엔티티클래스);
        } catch (NoSuchIndexException e) {
            String errorMessage = e.getMessage();
            if (errorMessage != null && errorMessage.contains("no such index")) {
                throw new IllegalArgumentException(e.getMessage());
            }
            throw e;
        } catch (Exception e){
            throw new IllegalArgumentException(e.getMessage());
        }
    }

    @Override
    public T findRecentDocById(ID id) {

        try{

            String recentFieldName = fieldInfo(엔티티클래스, Recent.class).getName();

            String idFieldName = fieldInfo(엔티티클래스,Id.class).getName();

            EsQuery esQuery
                = new EsQueryBuilder()
                    .bool(
                        new TermQueryFilter(idFieldName, id),
                        new TermQueryFilter(recentFieldName,true)
                    );

            ElasticSearchIndex annotation = AnnotationUtils.findAnnotation(엔티티클래스, ElasticSearchIndex.class);

            if(annotation==null){
                log.warn("ElasticSearchIndex annotatoin is null");
            }

            DocumentSearchResult<T> documentSearchResult
                    = new DocumentSearchResult<>(엘라스틱서치오퍼레이션스.search(기본_쿼리_생성기.기본검색(esQuery).생성(), 엔티티클래스),엔티티클래스);
            return documentSearchResult.fetchOnlyOne();

        } catch (NoSuchIndexException e) {
            String errorMessage = e.getMessage();
            if (errorMessage != null && errorMessage.contains("no such index")) {
                throw new IllegalArgumentException(e.getMessage());
            }
            throw e;
        } catch (Exception e){
            throw new IllegalArgumentException(e.getMessage());
        }
    }

    @Override
    public T findDocById(ID id) {
        return 엘라스틱서치오퍼레이션스.get(String.valueOf(id),엔티티클래스);
    }

    @Override
    public <S extends T> S save(S entity, String indexName)  {
        return 엘라스틱서치오퍼레이션스.save(entity,IndexCoordinates.of(indexName));
    }

    @Override
    public <S extends T> Iterable<S> saveAll(Iterable<S> entities){

        return StreamSupport.stream(entities.spliterator(), false)
                .collect(toList()).stream()
                .map(entity-> save(entity)).collect(toList());
    }

    @Override
    public <S extends T> S  save(S entity){

        ElasticSearchIndex annotation = AnnotationUtils.findAnnotation(엔티티클래스, ElasticSearchIndex.class);

        if(annotation==null){
            return 엘라스틱서치오퍼레이션스.save(entity);
        }

        Iterable<S> s = this.saveSupport(List.of(entity));

        if(!s.iterator().hasNext()){
            log.info("저장할 데이터가 없습니다.");
        }

        return entity;
    }

    private  <S extends T> Iterable<S> saveSupport(Iterable<S> entities) {

        log.info("[ 공통저장소_구현체 :: saveSupport ] entities = {}",entities);

        //동일한 이슈 수집 데이터이지만, 이력 추적을 위한 용도로 필드를 추가함 ( recent = true )
        RecentFieldConvertor<S,T> recentFieldConvertor = null;

        try {
            recentFieldConvertor = new RecentFieldConvertor<>(entities,entityClass);
        } catch (IllegalStateException e) {
            log.info(e.getMessage());
            return entities;
        }

        List<S> recentTrueList = StreamSupport.stream(entities.spliterator(), false)
                .filter(Objects::nonNull)
                .map(recentFieldConvertor::recentTrue)
                .filter(Objects::nonNull)
                .collect(toList());

        Map<String, List<T>> recentFalseListIfNotEqual
            = recentFieldConvertor.listMapGroupByIndex(recentFieldConvertor::setRecentFalseIfNotEqual);

        Map<String, List<T>> recentFalseIfDuplicate
            = recentFieldConvertor.listMapGroupByIndex(recentFieldConvertor::setRecentFalseIfDuplicate);

        recentFalseListIfNotEqual
            .forEach((key, value) -> 엘라스틱서치오퍼레이션스.save(value, IndexCoordinates.of(key)));

        recentFalseIfDuplicate
            .forEach((key, value) -> 엘라스틱서치오퍼레이션스.save(value, IndexCoordinates.of(key)));

        return recentTrueList.stream()
            .map(doc-> 엘라스틱서치오퍼레이션스.save(doc, 인덱스_명())).collect(toList());
    }

    @Override
    public List<T> findRecentDocsByScrollApi(Query query) {

        NativeSearchQuery nativeSearchQuery = recentTrueQueryBuilder((NativeSearchQuery)query);
        ElasticsearchRestTemplate template = (ElasticsearchRestTemplate)엘라스틱서치오퍼레이션스;
        IndexCoordinates indexCoordinatesFor = template.getIndexCoordinatesFor(엔티티클래스);

        SearchScrollHits<T> scroll
            = template.searchScrollStart(60000, nativeSearchQuery, 엔티티클래스, indexCoordinatesFor);

        String scrollId = scroll.getScrollId();
        List<T> entities = new ArrayList<>();

        while (scroll.hasSearchHits()) {
            entities.addAll(
                scroll.getSearchHits()
                    .stream()
                    .map(SearchHit::getContent)
                    .collect(toList())
            );
            scrollId = scroll.getScrollId();
            scroll = template.searchScrollContinue(scrollId, 60000, 엔티티클래스,indexCoordinatesFor);
        }

        assert scrollId != null;
        template.searchScrollClear(List.of(scrollId));

        return entities;

    }

    @Override
    public Set<String> findIndexNamesByAlias(IndexCoordinates indexCoordinates) {
        IndexOperations indexOperations = 엘라스틱서치오퍼레이션스.indexOps(indexCoordinates);
        return indexOperations.getAliasesForIndex(indexCoordinates.getIndexName()).keySet();
    }

    @Override
    public boolean deleteIndex(IndexCoordinates indexCoordinates) {
        IndexOperations indexOperations = 엘라스틱서치오퍼레이션스.indexOps(indexCoordinates);
        return indexOperations.delete();
    }

    @Override
    public String deleteDocById(String id, String indexName) {
        IndexCoordinates indexCoordinates = IndexCoordinates.of(indexName);
        return 엘라스틱서치오퍼레이션스.delete(id, indexCoordinates);
    }

    @Override
    public String deleteDocByEntity(T t) {
        return 엘라스틱서치오퍼레이션스.delete(t);
    }

    @Override
    public boolean 인덱스_존재_확인(String 인덱스명) {
        IndexOperations 인덱스작업 = 엘라스틱서치오퍼레이션스.indexOps(IndexCoordinates.of(인덱스명));
        return 인덱스작업.exists();
    }

    @Override
    public boolean 인덱스삭제(String 삭제할_지라이슈인덱스) {
        boolean 삭제결과 = false;

        IndexOperations 인덱스작업 = 엘라스틱서치오퍼레이션스.indexOps(IndexCoordinates.of(삭제할_지라이슈인덱스));

        try {
            if (인덱스작업.exists()) {
                삭제결과 = 인덱스작업.delete();
            }
        }
        catch(Exception e) {
            log.error( "{} :: 인덱스삭제(String 삭제할_지라이슈인덱스) :: 삭제할_지라이슈인덱스 -> {}",this.getClass().getName(),삭제할_지라이슈인덱스);
            log.error( "{} :: 인덱스삭제(String 삭제할_지라이슈인덱스) :: 에러 메세지 -> {}",this.getClass().getName(),e.getMessage());
        }

        return 삭제결과;
    }

    private IndexCoordinates 인덱스_명() {
        Document document = AnnotationUtils.findAnnotation(엔티티클래스, Document.class);
        if(document!=null){

            IndexCoordinates indexCoordinatesFor = 엘라스틱서치오퍼레이션스.getIndexCoordinatesFor(엔티티클래스);

            String indexName = indexCoordinatesFor.getIndexName();

            Method method = methodInfo(엔티티클래스, RollingIndexName.class);
            if(method!=null){
                try{
                    Constructor<T> constructor = 엔티티클래스.getConstructor();
                    T t = constructor.newInstance();
                    return IndexCoordinates.of(indexName +"-"+method.invoke(t));
                }catch (Exception e) {
                    return IndexCoordinates.of(indexName);
                }
            }else{
                return IndexCoordinates.of(indexName);
            }
        }
        throw new IllegalArgumentException("인덱스명을 확인해주시길 바랍니다.");
    }

    private NativeSearchQuery recentTrueQueryBuilder(NativeSearchQuery query) {

        String recentFieldName = fieldInfo(엔티티클래스, Recent.class).getName();

        EsQuery esQuery
            = new EsQueryBuilder()
            .bool(
                new TermsQueryFilter(recentFieldName, true)
            );
        BoolQueryBuilder boolQuery = esQuery.getQuery(new ParameterizedTypeReference<>() {
        });

        BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery()
            .filter(boolQuery);

        Optional.ofNullable(query.getQuery()).ifPresent(boolQueryBuilder::filter);

        NativeSearchQueryBuilder nativeSearchQueryBuilder = new NativeSearchQueryBuilder()
            .withQuery(boolQueryBuilder)
            .withPageable(query.getPageable())
            .withMaxResults(Optional.ofNullable(query.getMaxResults()).orElse(0));

        Optional.ofNullable(query.getAggregations())
                .ifPresent(args-> args.forEach(nativeSearchQueryBuilder::addAggregation));

        return nativeSearchQueryBuilder.build();

    }

}
